@@ -34,11 +34,21 @@
 
 #define ALLOCATION_CHUNK_SIZE	16
 
+/*
+ * The implementation of astr.
+ */
+struct astr
+{
+  char *text;		/* The string buffer. */
+  size_t len;		/* The length of the string. */
+  size_t maxlen;	/* The buffer size. */
+};
+
 astr
 astr_new (void)
 {
   astr as;
-  as = (astr) xzalloc (sizeof *as);
+  as = (astr) XZALLOC (struct astr);
   as->maxlen = ALLOCATION_CHUNK_SIZE;
   as->len = 0;
   as->text = (char *) xzalloc (as->maxlen + 1);
@@ -46,6 +56,12 @@ astr_new (void)
   return as;
 }
 
+astr
+astr_new_cstr (const char *s)
+{
+  return astr_cpy_cstr (astr_new (), s);
+}
+
 static void
 astr_resize (astr as, size_t reqsize)
 {
@@ -68,7 +84,7 @@ astr_pos (astr as, ptrdiff_t pos)
 }
 
 char *
-astr_char (const astr as, ptrdiff_t pos)
+astr_char (astr as, ptrdiff_t pos)
 {
   assert (as != NULL);
   pos = astr_pos (as, pos);
@@ -84,32 +100,42 @@ astr_delete (astr as)
   free (as);
 }
 
-static astr
-astr_cpy_x (astr as, const char *s, size_t csize)
+const char *
+astr_cstr (astr as)
 {
-  astr_resize (as, csize);
-  memcpy (as->text, s, csize);
-  as->len = csize;
-  as->text[csize] = '\0';
+  return (const char *) (as->text);
+}
+
+size_t
+astr_len (astr as)
+{
+  return as->len;
+}
+
+astr
+astr_ncpy_cstr (astr as, const char *s, size_t len)
+{
+  astr_resize (as, len);
+  memcpy (as->text, s, len);
+  as->len = len;
+  as->text[len] = '\0';
   return as;
 }
 
 astr
-astr_cpy (astr as, const astr src)
+astr_cpy (astr as, astr src)
 {
-  assert (src != NULL);
-  return astr_cpy_x (as, src->text, src->len);
+  return astr_ncpy_cstr (as, src->text, src->len);
 }
 
 astr
 astr_cpy_cstr (astr as, const char *s)
 {
-  assert (s != NULL);
-  return astr_cpy_x (as, s, strlen (s));
+  return astr_ncpy_cstr (as, s, strlen (s));
 }
 
-static astr
-astr_cat_x (astr as, const char *s, size_t csize)
+astr
+astr_ncat_cstr (astr as, const char *s, size_t csize)
 {
   astr_resize (as, as->len + csize);
   memcpy (as->text + as->len, s, csize);
@@ -119,16 +145,9 @@ astr_cat_x (astr as, const char *s, size_t csize)
 }
 
 astr
-astr_cat (astr as, const astr src)
+astr_cat (astr as, astr src)
 {
-  assert (src != NULL);
-  return astr_cat_x (as, src->text, src->len);
-}
-
-astr
-astr_ncat_cstr (astr as, const char *s, size_t len)
-{
-  return astr_cat_x (as, s, len);
+  return astr_ncat_cstr (as, src->text, src->len);
 }
 
 astr
@@ -148,16 +167,16 @@ astr_cat_char (astr as, int c)
 }
 
 astr
-astr_cat_delete (astr as, const astr src)
+astr_cat_delete (astr as, astr src)
 {
   assert (src != NULL);
-  astr_cat_x (as, src->text, src->len);
+  astr_cat (as, src);
   astr_delete (src);
   return as;
 }
 
 astr
-astr_substr (const astr as, ptrdiff_t pos, size_t size)
+astr_substr (astr as, ptrdiff_t pos, size_t size)
 {
   assert (as != NULL);
   pos = astr_pos (as, pos);
@@ -166,38 +185,14 @@ astr_substr (const astr as, ptrdiff_t pos, size_t size)
 }
 
 int
-astr_find (const astr as, const astr src)
+astr_cmp(astr as1, astr as2)
 {
-  return astr_find_cstr (as, src->text);
-}
-
-int
-astr_find_cstr (const astr as, const char *s)
-{
-  char *sp;
-  assert (as != NULL && s != NULL);
-  sp = strstr (as->text, s);
-  return (sp == NULL) ? -1 : sp - as->text;
-}
-
-int
-astr_rfind (const astr as, const astr src)
-{
-  return astr_rfind_cstr (as, src->text);
-}
-
-int
-astr_rfind_cstr (const astr as, const char *s)
-{
-  char *sp;
-  assert (as != NULL && s != NULL);
-  sp = strrstr (as->text, s);
-  return (sp == NULL) ? -1 : sp - as->text;
+  return strcmp (as1->text, as2->text);
 }
 
 static astr
 astr_replace_x (astr as, ptrdiff_t pos, size_t size, const char *s,
-		size_t csize)
+                size_t csize)
 {
   astr tail;
 
@@ -216,13 +211,6 @@ astr_replace_x (astr as, ptrdiff_t pos, size_t size, const char *s,
   return as;
 }
 
-astr
-astr_replace (astr as, ptrdiff_t pos, size_t size, const astr src)
-{
-  assert (src != NULL);
-  return astr_replace_x (as, pos, size, src->text, src->len);
-}
-
 astr
 astr_replace_cstr (astr as, ptrdiff_t pos, size_t size, const char *s)
 {
@@ -231,23 +219,10 @@ astr_replace_cstr (astr as, ptrdiff_t pos, size_t size, const char *s)
 }
 
 astr
-astr_replace_char (astr as, ptrdiff_t pos, size_t size, int c)
+astr_replace_char (astr as, ptrdiff_t pos, int c)
 {
-  return astr_replace_x (as, pos, size, (const char *) &c, (size_t) 1);
-}
-
-astr
-astr_insert (astr as, ptrdiff_t pos, const astr src)
-{
-  assert (src != NULL);
-  return astr_replace_x (as, pos, (size_t) 0, src->text, src->len);
-}
-
-astr
-astr_insert_cstr (astr as, ptrdiff_t pos, const char *s)
-{
-  assert (s != NULL);
-  return astr_replace_x (as, pos, (size_t) 0, s, strlen (s));
+  char ch = (char) c;
+  return astr_replace_x (as, pos, (size_t) 1, &ch, (size_t) 1);
 }
 
 astr
@@ -277,16 +252,27 @@ astr_truncate (astr as, ptrdiff_t pos)
   return as;
 }
 
+astr astr_fread (FILE * fp)
+{
+  int c;
+  astr as = astr_new ();
+
+  while ((c = getc (fp)) != EOF)
+    astr_cat_char (as, c);
+
+  return as;
+}
+
 astr
-astr_fgets (FILE * f)
+astr_fgets (FILE * fp)
 {
   int c;
   astr as;
 
-  if (feof (f))
+  if (feof (fp))
     return NULL;
   as = astr_new ();
-  while ((c = fgetc (f)) != EOF && c != '\n')
+  while ((c = fgetc (fp)) != EOF && c != '\n')
     astr_cat_char (as, c);
   return as;
 }
@@ -318,7 +304,7 @@ astr_afmt (astr as, const char *fmt, ...)
 static void
 assert_eq (astr as, const char *s)
 {
-  if (astr_cmp_cstr (as, s))
+  if (strcmp (((astr)(as))->text, (s)))
     printf ("test failed: \"%s\" != \"%s\"\n", as->text, s);
 }
 
@@ -338,7 +324,6 @@ int
 main (void)
 {
   astr as1, as2, as3;
-  int i;
   FILE *fp;
 
   as1 = astr_new ();
@@ -362,12 +347,6 @@ main (void)
   astr_cpy_cstr (as1, "12345");
   astr_delete (as2);
 
-  astr_cpy_cstr (as1, "12345");
-  astr_insert_cstr (as1, 3, "mid");
-  astr_insert_cstr (as1, 0, "begin");
-  astr_cat_cstr (as1, "end");
-  assert_eq (as1, "begin123mid45end");
-
   astr_cpy_cstr (as1, "12345");
   astr_insert_char (as1, -2, 'x');
   astr_insert_char (as1, -6, 'y');
@@ -390,17 +369,6 @@ main (void)
   astr_remove (as1, 4, 10);
   assert_eq (as1, "1234");
 
-  astr_cpy_cstr (as1, "abc def de ab cd ab de fg");
-  while ((i = astr_find_cstr (as1, "de")) >= 0)
-    astr_replace_cstr (as1, i, 2, "xxx");
-  assert_eq (as1, "abc xxxf xxx ab cd ab xxx fg");
-  while ((i = astr_find_cstr (as1, "ab")) >= 0)
-    astr_remove (as1, i, 2);
-  assert_eq (as1, "c xxxf xxx  cd  xxx fg");
-  while ((i = astr_find_cstr (as1, "  ")) >= 0)
-    astr_replace_char (as1, i, 2, ' ');
-  assert_eq (as1, "c xxxf xxx cd xxx fg");
-
   astr_cpy_cstr (as1, "12345");
   as2 = astr_substr (as1, -2, 2);
   assert_eq (as2, "45");
@@ -422,11 +390,6 @@ main (void)
   astr_replace_cstr (as1, -1, 5, "foo");
   assert_eq (as1, "123456foo");
 
-  astr_cpy_cstr (as1, "abc def de ab cd ab de fg");
-  while ((i = astr_find_cstr (as1, "de")) >= 0)
-    astr_replace_cstr (as1, i, 2, "xxx");
-  assert_eq (as1, "abc xxxf xxx ab cd ab xxx fg");
-
   astr_cpy_cstr (as1, "");
   astr_afmt (as1, "%s * %d = ", "5", 3);
   astr_afmt (as1, "%d", 15);
