@@ -43,17 +43,17 @@ buffer_new (void)
 {
   Buffer *bp;
 
-  bp = (Buffer *) xzalloc (sizeof (Buffer));
+  bp = (Buffer *) XZALLOC (Buffer);
 
   /* Allocate a line. */
-  bp->pt.p = list_new ();
-  bp->pt.p->item = astr_new ();
+  bp->pt.p = line_new ();
+  bp->pt.p->text = astr_new ();
 
   /* Allocate the limit marker. */
-  bp->lines = list_new ();
+  bp->lines = line_new ();
 
-  list_prev (bp->lines) = list_next (bp->lines) = bp->pt.p;
-  list_prev (bp->pt.p) = list_next (bp->pt.p) = bp->lines;
+  bp->lines->prev = bp->lines->next = bp->pt.p;
+  bp->pt.p->prev = bp->pt.p->next = bp->lines;
 
   /* Set default EOL string. */
   bp->eol = coding_eol_lf;
@@ -62,38 +62,22 @@ buffer_new (void)
 }
 
 /*
- * Free the buffer allocated memory.
+ * Free the buffer's allocated memory.
  */
 void
 free_buffer (Buffer * bp)
 {
-  Line *lp;
-  Undo *up, *next_up;
+  line_delete (bp->lines);
+  free_undo (bp->last_undop);
 
-  /* Free all the lines. */
-  for (lp = list_first (bp->lines); lp != bp->lines; lp = list_next (lp))
-    astr_delete (lp->item);
-  list_delete (bp->lines);
-
-  /* Free all the undo operations. */
-  up = bp->last_undop;
-  while (up != NULL)
-    {
-      next_up = up->next;
-      if (up->type == UNDO_INSERT_BLOCK || up->type == UNDO_REPLACE_BLOCK)
-	free (up->delta.block.text);
-      free (up);
-      up = next_up;
-    }
-
-  /* Free markers. */
   while (bp->markers)
     free_marker (bp->markers);
 
-  /* Free the name and the filename. */
   free (bp->name);
-  if (bp->filename != NULL)
-    free (bp->filename);
+  free (bp->filename);
+
+  if (bp->vars != NULL)
+    hash_free (bp->vars);
 
   free (bp);
 }
@@ -119,7 +103,7 @@ free_buffers (void)
 void
 init_buffer (Buffer * bp)
 {
-  if (lookup_bool_variable ("auto-fill-mode"))
+  if (get_variable_bool ("auto-fill-mode"))
     bp->flags ^= BFLAG_AUTOFILL;
 }
 
@@ -148,8 +132,7 @@ create_buffer (const char *name)
 void
 set_buffer_name (Buffer * bp, const char *name)
 {
-  if (bp->name != NULL)
-    free (bp->name);
+  free (bp->name);
   bp->name = xstrdup (name);
 }
 
@@ -159,12 +142,10 @@ set_buffer_name (Buffer * bp, const char *name)
 void
 set_buffer_filename (Buffer * bp, const char *filename)
 {
-  if (bp->filename != NULL)
-    free (bp->filename);
+  free (bp->filename);
   bp->filename = xstrdup (filename);
 
-  if (bp->name != NULL)
-    free (bp->name);
+  free (bp->name);
   bp->name = "";
   bp->name = make_buffer_name (filename);
 }
@@ -197,15 +178,6 @@ find_buffer (const char *name, int cflag)
   return bp;
 }
 
-/*
- * Find the next buffer in buffer list.
- */
-Buffer *
-get_next_buffer (void)
-{
-  return cur_bp->next ? cur_bp->next : head_bp;
-}
-
 /*
  * Create a buffer name using the file name.
  */
@@ -234,14 +206,11 @@ make_buffer_name (const char *filename)
     {
       sprintf (name, "%s<%d>", p, i);
       if (find_buffer (name, false) == NULL)
-	return name;
+        return name;
     }
 
-  /*
-   * This should never happen.
-   */
-  assert (0);
-  return NULL;
+   /* This should never happen. */
+  abort ();
 }
 
 /* Move the selected buffer to head.  */
@@ -254,15 +223,15 @@ move_buffer_to_head (Buffer * bp)
   for (it = head_bp; it; it = it->next)
     {
       if (bp == it)
-	{
-	  if (prev)
-	    {
-	      prev->next = bp->next;
-	      bp->next = head_bp;
-	      head_bp = bp;
-	    }
-	  break;
-	}
+        {
+          if (prev)
+            {
+              prev->next = bp->next;
+              bp->next = head_bp;
+              head_bp = bp;
+            }
+          break;
+        }
       prev = it;
     }
 }
@@ -304,7 +273,7 @@ warn_if_readonly_buffer (void)
   return false;
 }
 
-int
+static int
 warn_if_no_mark (void)
 {
   if (!cur_bp->mark)
@@ -312,7 +281,7 @@ warn_if_no_mark (void)
       minibuf_error ("The mark is not set now");
       return true;
     }
-  else if (transient_mark_mode () && !cur_bp->mark_active)
+  else if (!(cur_bp->flags & BFLAG_MARK) && transient_mark_mode ())
     {
       minibuf_error ("The mark is not active now");
       return true;
@@ -321,29 +290,6 @@ warn_if_no_mark (void)
     return false;
 }
 
-/*
- * Calculate a region size and set the region structure.
- */
-void
-calculate_region (Region * rp, Point from, Point to)
-{
-  if (cmp_point (from, to) < 0)
-    {
-      /* The point is before the mark. */
-      rp->start = from;
-      rp->end = to;
-    }
-  else
-    {
-      /* The mark is before the point. */
-      rp->start = to;
-      rp->end = from;
-    }
-
-  rp->size = point_dist (rp->start, rp->end);
-  rp->num_lines = count_lines (rp->start, rp->end);
-}
-
 /*
  * Calculate the region size between point and mark and set the region
    structure.
@@ -351,10 +297,23 @@ calculate_region (Region * rp, Point from, Point to)
 int
 calculate_the_region (Region * rp)
 {
-  if (!is_mark_actived ())
+  if (warn_if_no_mark ())
     return false;
 
-  calculate_region (rp, cur_bp->pt, cur_bp->mark->pt);
+  if (cmp_point (cur_bp->pt, cur_bp->mark->pt) < 0)
+    {
+      /* Point is before mark. */
+      rp->start = cur_bp->pt;
+      rp->end = cur_bp->mark->pt;
+    }
+  else
+    {
+      /* Mark is before point. */
+      rp->start = cur_bp->mark->pt;
+      rp->end = cur_bp->pt;
+    }
+
+  rp->size = point_dist (rp->start, rp->end);
   return true;
 }
 
@@ -372,7 +331,7 @@ set_temporary_buffer (Buffer * bp)
   if (bp == head_bp)
     {
       if (head_bp->next == NULL)
-	return;
+        return;
       head_bp = head_bp->next;
     }
   else if (bp->next == NULL)
@@ -381,8 +340,8 @@ set_temporary_buffer (Buffer * bp)
   for (bp0 = head_bp; bp0 != NULL; bp0 = bp0->next)
     if (bp0->next == bp)
       {
-	bp0->next = bp0->next->next;
-	break;
+        bp0->next = bp0->next->next;
+        break;
       }
 
   for (bp0 = head_bp; bp0->next != NULL; bp0 = bp0->next)
@@ -395,7 +354,7 @@ set_temporary_buffer (Buffer * bp)
 size_t
 calculate_buffer_size (Buffer * bp)
 {
-  Line *lp = list_next (bp->lines);
+  Line *lp = bp->lines->next;
   size_t size = 0;
 
   if (lp == bp->lines)
@@ -403,10 +362,10 @@ calculate_buffer_size (Buffer * bp)
 
   for (;;)
     {
-      size += astr_len (lp->item);
-      lp = list_next (lp);
+      size += astr_len (lp->text);
+      lp = lp->next;
       if (lp == bp->lines)
-	break;
+        break;
       ++size;
     }
 
@@ -416,30 +375,19 @@ calculate_buffer_size (Buffer * bp)
 int
 transient_mark_mode (void)
 {
-  return lookup_bool_variable ("transient-mark-mode");
+  return get_variable_bool ("transient-mark-mode");
 }
 
 void
 activate_mark (void)
 {
-  cur_bp->mark_active = true;
+  cur_bp->flags |= BFLAG_MARK;
 }
 
 void
 deactivate_mark (void)
 {
-  cur_bp->mark_active = false;
-}
-
-int
-is_mark_actived (void)
-{
-  if (!cur_bp->mark)
-    return false;
-  else if (transient_mark_mode ())
-    return (cur_bp->mark_active) ? true : false;
-  else
-    return true;
+  cur_bp->flags &= ~BFLAG_MARK;
 }
 
 /*
@@ -452,3 +400,49 @@ tab_width (Buffer * bp)
 
   return t ? t : 1;
 }
+
+/*
+ * Copy a region of text into an allocated buffer.
+ */
+char *
+copy_text_block (size_t startn, size_t starto, size_t size)
+{
+  char *buf, *dp;
+  size_t max_size, n, i;
+  Line *lp;
+
+  max_size = 10;
+  dp = buf = (char *) xzalloc (max_size);
+
+  lp = cur_bp->pt.p;
+  n = cur_bp->pt.n;
+  if (n > startn)
+    do
+      lp = lp->prev;
+    while (--n > startn);
+  else if (n < startn)
+    do
+      lp = lp->next;
+    while (++n < startn);
+
+  for (i = starto; dp - buf < (int) size;)
+    {
+      if (dp >= buf + max_size)
+        {
+          int save_off = dp - buf;
+          max_size += 10;
+          buf = (char *) xrealloc (buf, max_size);
+          dp = buf + save_off;
+        }
+      if (i < astr_len (lp->text))
+        *dp++ = *astr_char (lp->text, (ptrdiff_t) (i++));
+      else
+        {
+          *dp++ = '\n';
+          lp = lp->next;
+          i = 0;
+        }
+    }
+
+  return buf;
+}
