@@ -22,38 +22,30 @@
    Free Software Foundation, Fifth Floor, 51 Franklin Street, Boston,
    MA 02111-1301, USA.  */
 
-#include "eval.h"
+#include "gl_list.h"
 
 /* basic.c ---------------------------------------------------------------- */
 size_t get_goalc_bp (Buffer * bp, Point pt);
-size_t get_goalc_wp (Window * wp);
 size_t get_goalc (void);
 int backward_char (void);
 int forward_char (void);
-void goto_line (size_t to_line);
 void gotobob (void);
 void gotoeob (void);
 int next_line (void);
 int ngotodown (size_t n);
 int ngotoup (size_t n);
 int previous_line (void);
-int scroll_down (void);
-int scroll_up (void);
 
 /* bind.c ----------------------------------------------------------------- */
+int completion_strcmp (const void *p1, const void *p2);
 size_t do_completion (astr as);
-char *minibuf_read_function_name (const char *fmt, ...);
-int execute_function (char *name, int uniarg);
-char *get_function_by_key_sequence (size_t ** keys, int *numkeys);
+const char *get_function_by_key_sequence (gl_list_t * keys);
 void process_key (size_t key);
 void init_bindings (void);
 Function last_command (void);
 void free_bindings (void);
-Function get_function (char *name);
-char *get_function_name (Function p);
 
 /* buffer.c --------------------------------------------------------------- */
-void calculate_region (Region * rp, Point from, Point to);
 int calculate_the_region (Region * rp);
 void init_buffer (Buffer * bp);
 Buffer *create_buffer (const char *name);
@@ -62,18 +54,16 @@ void free_buffers (void);
 void set_buffer_name (Buffer * bp, const char *name);
 void set_buffer_filename (Buffer * bp, const char *filename);
 Buffer *find_buffer (const char *name, int cflag);
-Buffer *get_next_buffer (void);
 char *make_buffer_name (const char *filename);
 void switch_to_buffer (Buffer * bp);
 int warn_if_readonly_buffer (void);
-int warn_if_no_mark (void);
 void set_temporary_buffer (Buffer * bp);
 size_t calculate_buffer_size (Buffer * bp);
 int transient_mark_mode (void);
 void activate_mark (void);
 void deactivate_mark (void);
-int is_mark_actived (void);
 size_t tab_width (Buffer * bp);
+char *copy_text_block (size_t startn, size_t starto, size_t size);
 
 /* completion.c ----------------------------------------------------------- */
 Completion *completion_new (int fileflag);
@@ -88,92 +78,87 @@ void pop_mark (void);
 void set_mark (void);
 int is_empty_line (void);
 int is_blank_line (void);
-int char_after (Point * pt);
-int char_before (Point * pt);
 int following_char (void);
 int preceding_char (void);
 int bobp (void);
 int eobp (void);
 int bolp (void);
 int eolp (void);
+void ding (void);
+
+/* eval.c ----------------------------------------------------------------- */
+extern le *leNIL, *leT;
+size_t countNodes (le * branch);
+void leEval (le * list);
+le *execute_with_uniarg (bool undo, int uniarg, int (*forward) (void),
+                         int (*backward) (void));
+le *execute_function (const char *name, int uniarg);
+Function get_function (const char *name);
+const char *get_function_doc (const char *name);
+const char *get_function_name (Function p);
+const char *minibuf_read_function_name (const char *fmt, ...);
+void init_eval (void);
+void free_eval (void);
 
 /* file.c ----------------------------------------------------------------- */
 int exist_file (const char *filename);
-int is_regular_file (const char *filename);
-astr agetcwd (void);
 astr get_home_dir (void);
 astr expand_path (astr path);
-astr compact_path (const astr path);
-void read_from_disk (const char *filename);
+astr compact_path (astr path);
 int find_file (const char *filename);
-Completion *make_buffer_completion (void);
-int check_modified_buffer (Buffer * bp);
 void kill_buffer (Buffer * kill_bp);
 void zile_exit (int doabort);
 
 /* funcs.c ---------------------------------------------------------------- */
-int cancel (void);
-int set_mark_command (void);
-int exchange_point_and_mark (void);
-int universal_argument (int keytype, int xarg);
-void write_temp_buffer (const char *name, void (*func) (va_list ap), ...);
-int forward_sexp (void);
-int backward_sexp (void);
+void set_mark_interactive (void);
+le *universal_argument (int keytype, int xarg);
+void write_temp_buffer (const char *name, bool show, void (*func) (va_list ap), ...);
+bool delete_region (const Region * r);
 
-/* glue.c ----------------------------------------------------------------- */
-void ding (void);
+/* getkey.c --------------------------------------------------------------- */
 void ungetkey (size_t key);
 size_t lastkey (void);
 size_t xgetkey (int mode, size_t timeout);
 size_t getkey (void);
 void waitkey (size_t delay);
-char *copy_text_block (size_t startn, size_t starto, size_t size);
-astr shorten_string (char *s, int maxlen);
-char *replace_string (char *s, char *match, char *subst);
-void tabify_string (char *dest, char *src, size_t scol, size_t tw);
-void untabify_string (char *dest, char *src, size_t scol, size_t tw);
-void goto_point (Point pt);
-char *getln (FILE * fp);
 
 /* history.c -------------------------------------------------------------- */
-void free_history_elements (History * hp);
+History *history_new (void);
+void free_history (History * hp);
 void add_history_element (History * hp, const char *string);
 void prepare_history (History * hp);
 const char *previous_history_element (History * hp);
 const char *next_history_element (History * hp);
 
-/* keys.c ----------------------------------------------------------------- */
+/* keycode.c -------------------------------------------------------------- */
 astr chordtostr (size_t key);
-size_t strtochord (char *buf, size_t * len);
-int keystrtovec (char *key, size_t ** keyvec);
-astr keyvectostr (size_t * keys, size_t numkeys);
-astr simplify_key (char *key);
+gl_list_t keystrtovec (char *key);
+astr keyvectostr (gl_list_t keys);
 
-/* line.c ----------------------------------------------------------------- */
-void line_replace_text (Line ** lp, size_t offset, size_t oldlen,
-			char *newtext, size_t newlen, int replace_case);
-int insert_char (int c);
-int insert_char_in_insert_mode (int c);
-int intercalate_char (int c);
-int insert_tab (void);
-void fill_break_line (void);
-int insert_newline (void);
-int intercalate_newline (void);
-void insert_string (const char *s);
-void insert_nstring (const char *s, size_t size);
-void bprintf (const char *fmt, ...);
-int delete_char (void);
-int backward_delete_char (void);
-void free_registers (void);
+/* killring.c ------------------------------------------------------------- */
 void free_kill_ring (void);
 
+/* line.c ----------------------------------------------------------------- */
+Line *line_new (void);
+void line_delete (Line *l);
+Line *line_insert (Line *l, astr i);
+void line_remove (Line *l);
+void line_replace_text (Line ** lp, size_t offset, size_t oldlen,
+                        char *newtext, size_t newlen, int replace_case);
+int insert_char (int c);
+int insert_char_in_insert_mode (int c);
+bool fill_break_line (void);
+int insert_newline (void);
+void insert_nstring (const char *s, size_t len);
+void insert_astr (astr as);
+void bprintf (const char *fmt, ...);
+int delete_char (void);
+void free_registers (void);
+
 /* lisp.c ----------------------------------------------------------------- */
-void lisp_init (void);
-void lisp_finalise (void);
-le *lisp_read (getcCallback getcp, ungetcCallback ungetcp);
-le *lisp_read_string (const char *string);
-le *lisp_read_file (const char *file);
-astr lisp_dump (le * list);
+void init_lisp (void);
+void free_lisp (void);
+bool lisp_load (const char *file);
 
 /* macro.c ---------------------------------------------------------------- */
 void cancel_kbd_macro (void);
@@ -181,17 +166,16 @@ void add_cmd_to_macro (void);
 void add_key_to_cmd (size_t key);
 void call_macro (Macro * mp);
 void free_macros (void);
-Macro *get_macro (char *name);
+Macro *get_macro (const char *name);
+void add_macros_to_list (gl_list_t l, gl_listelement_compar_fn f);
 
 /* main.c ----------------------------------------------------------------- */
 extern char *prog_name;
 extern Window *cur_wp, *head_wp;
 extern Buffer *cur_bp, *head_bp;
-extern Terminal *cur_tp;
 extern int thisflag, lastflag, last_uniarg;
 
 /* marker.c --------------------------------------------------------------- */
-Marker *make_marker (void);
 void free_marker (Marker * marker);
 void move_marker (Marker * marker, Buffer * bp, Point pt);
 Marker *copy_marker (Marker * marker);
@@ -200,14 +184,22 @@ Marker *point_min_marker (void);
 
 /* minibuf.c -------------------------------------------------------------- */
 extern char *minibuf_contents;
+void init_minibuf (void);
 void free_minibuf (void);
 void minibuf_error (const char *fmt, ...);
 void minibuf_write (const char *fmt, ...);
 char *minibuf_read (const char *fmt, const char *value, ...);
+size_t minibuf_read_number (const char *fmt, ...);
+bool minibuf_test_in_completions (const char *ms, gl_list_t completions);
 int minibuf_read_yesno (const char *fmt, ...);
-char *minibuf_read_dir (const char *fmt, const char *value, ...);
 char *minibuf_read_completion (const char *fmt, char *value, Completion * cp,
-			       History * hp, ...);
+                               History * hp, ...);
+char *vminibuf_read_completion (const char *fmt, char *value, Completion * cp,
+                                History * hp, const char *empty_err,
+                                bool (*test) (const char *s, gl_list_t completions),
+                                const char *invalid_err, va_list ap);
+char *minibuf_read_filename (const char *fmt, const char *value,
+                             const char *file, ...);
 void minibuf_clear (void);
 
 /* point.c ---------------------------------------------------------------- */
@@ -220,6 +212,7 @@ Point point_min (void);
 Point point_max (void);
 Point line_beginning_position (int count);
 Point line_end_position (int count);
+void goto_point (Point pt);
 
 /* redisplay.c ------------------------------------------------------------ */
 void resync_redisplay (void);
@@ -231,13 +224,10 @@ void init_search (void);
 
 /* term_minibuf.c --------------------------------------------------------- */
 void term_minibuf_write (const char *fmt);
-char *term_minibuf_read (const char *prompt, const char *value,
-			 Completion * cp, History * hp);
-void free_rotation_buffers (void);
+char *term_minibuf_read (const char *prompt, const char *value, size_t pos,
+                         Completion * cp, History * hp);
 
 /* term_redisplay.c ------------------------------------------------------- */
-int term_initted (void);
-void term_set_initted (void);
 size_t term_width (void);
 size_t term_height (void);
 void term_set_size (size_t cols, size_t rows);
@@ -247,7 +237,7 @@ void show_splash_screen (const char *splash);
 void term_tidy (void);
 void term_addnstr (const char *s, size_t len);
 
-/* term_termcap.c --------------------------------------------------------- */
+/* term_ncurses.c --------------------------------------------------------- */
 void term_init (void);
 void term_close (void);
 void term_suspend (void);
@@ -255,7 +245,6 @@ void term_resume (void);
 void term_move (size_t y, size_t x);
 void term_clrtoeol (void);
 void term_refresh (void);
-void term_redraw_cursor (void);
 void term_clear (void);
 void term_addch (int c);
 void term_attrset (size_t attrs, ...);
@@ -264,25 +253,26 @@ size_t term_xgetkey (int mode, size_t timeout);
 
 /* undo.c ----------------------------------------------------------------- */
 extern int undo_nosave;
-
 void undo_start_sequence (void);
 void undo_end_sequence (void);
 void undo_save (int type, Point pt, size_t arg1, size_t arg2);
+void free_undo (Undo *up);
+void undo_set_unchanged (Undo *up);
 
 /* variables.c ------------------------------------------------------------ */
 void init_variables (void);
-int is_variable_equal (char *var, char *val);
-int lookup_bool_variable (char *var);
+void free_variables (void);
+bool get_variable_bool (char *var);
 char *minibuf_read_variable_name (char *msg);
-void set_variable (char *var, char *val);
-char *get_variable_bp (Buffer * bp, char *var);
-char *get_variable (char *var);
+void set_variable (const char *var, const char *val);
+const char *get_variable_doc (Buffer * bp, char *var, char **defval);
+const char *get_variable_bp (Buffer * bp, char *var);
+const char *get_variable (char *var);
 int get_variable_number_bp (Buffer * bp, char *var);
 int get_variable_number (char *var);
 
 /* window.c --------------------------------------------------------------- */
 void create_first_window (void);
-void free_window (Window * wp);
 Window *find_window (const char *name);
 void free_windows (void);
 Window *popup_window (void);
@@ -291,24 +281,15 @@ void delete_window (Window * del_wp);
 Point window_pt (Window * wp);
 
 /* xalloc.c --------------------------------------------------------------- */
-#include "xalloc.h"
 int xvasprintf (char **ptr, const char *fmt, va_list vargs);
 int xasprintf (char **ptr, const char *fmt, ...);
+void list_free (const void *p);
 
 
 /*
  * Declare external Zile functions.
  */
-#define X0(zile_name, c_name)			\
-	extern int F_ ## c_name(int uniarg, le *l);
-#define X1(zile_name, c_name, k1)		\
-	X0(zile_name, c_name)
-#define X2(zile_name, c_name, k1, k2)		\
-	X0(zile_name, c_name)
-#define X3(zile_name, c_name, k1, k2, k3)	\
-	X0(zile_name, c_name)
+#define X(zile_name, c_name, doc)                \
+  extern le *F_ ## c_name (int uniarg, le * l);
 #include "tbl_funcs.h"
-#undef X0
-#undef X1
-#undef X2
-#undef X3
+#undef X
