@@ -24,9 +24,6 @@
 #include "config.h"
 
 #include <sys/stat.h>
-#if HAVE_SYS_TYPES_H
-#include <sys/types.h>
-#endif
 #include <assert.h>
 #include <errno.h>
 #include <fcntl.h>
@@ -36,8 +33,8 @@
 #include <stdlib.h>
 #include <string.h>
 #include <unistd.h>
-#include "euidaccess.h"
 #include <utime.h>
+#include "dirname.h"
 
 #include "zile.h"
 #include "extern.h"
@@ -54,7 +51,7 @@ exist_file (const char *filename)
   return true;
 }
 
-int
+static int
 is_regular_file (const char *filename)
 {
   struct stat st;
@@ -62,7 +59,7 @@ is_regular_file (const char *filename)
   if (stat (filename, &st) == -1)
     {
       if (errno == ENOENT)
-	return true;
+        return true;
       return false;
     }
   if (S_ISREG (st.st_mode))
@@ -71,26 +68,15 @@ is_regular_file (const char *filename)
   return false;
 }
 
-/* Safe getcwd */
-astr
+/*
+ * Return the current directory, if available.
+ */
+static astr
 agetcwd (void)
 {
-  size_t len = PATH_MAX;
-  char *buf = (char *) xzalloc (len);
-  char *res;
-  astr as;
-
-  while ((res = getcwd (buf, len)) == NULL && errno == ERANGE)
-    {
-      len *= 2;
-      buf = xrealloc (buf, len);
-    }
-  /* If there was an error, return the empty string */
-  if (res == NULL)
-    *buf = '\0';
-  as = astr_new_cstr (buf);
-  free (buf);
-
+  char *s = getcwd (NULL, 0);
+  astr as = astr_new_cstr (s);
+  free (s);
   return as;
 }
 
@@ -101,7 +87,7 @@ agetcwd (void)
  * - replaces `//' with `/' (restarting from the root directory);
  * - removes `..' and `.' entries.
  *
- * If something goes wrong, the string is deleted and NULL returned
+ * If something goes wrong, the string is deleted and NULL returned.
  */
 astr
 expand_path (astr path)
@@ -115,98 +101,105 @@ expand_path (astr path)
     {
       astr_cat_delete (epath, agetcwd ());
       if (astr_len (epath) == 0 || *astr_char (epath, -1) != '/')
-	astr_cat_char (epath, '/');
+        astr_cat_char (epath, '/');
     }
 
   while (*sp != '\0')
     {
       if (*sp == '/')
-	{
-	  if (*++sp == '/')
-	    {
-	      /* Got `//'.  Restart from this point. */
-	      while (*sp == '/')
-		sp++;
-	      astr_truncate (epath, 0);
-	    }
-	  astr_cat_char (epath, '/');
-	}
+        {
+          if (*++sp == '/')
+            {
+              /* Got `//'.  Restart from this point. */
+              while (*sp == '/')
+                sp++;
+              astr_truncate (epath, 0);
+            }
+          astr_cat_char (epath, '/');
+        }
       else if (*sp == '~')
-	{
-	  if (*(sp + 1) == '/')
-	    {
-	      /* Got `~/'. Restart from this point and insert the user's
-		 home directory. */
-	      astr_truncate (epath, 0);
-	      pw = getpwuid (getuid ());
-	      if (pw == NULL)
-		{
-		  ret = false;
-		  break;
-		}
-	      if (strcmp (pw->pw_dir, "/") != 0)
-		astr_cat_cstr (epath, pw->pw_dir);
-	      ++sp;
-	    }
-	  else
-	    {
-	      /* Got `~something'.  Restart from this point and insert that
-		 user's home directory. */
-	      astr as = astr_new ();
-	      astr_truncate (epath, 0);
-	      ++sp;
-	      while (*sp != '\0' && *sp != '/')
-		astr_cat_char (as, *sp++);
-	      pw = getpwnam (astr_cstr (as));
-	      astr_delete (as);
-	      if (pw == NULL)
-		{
-		  ret = false;
-		  break;
-		}
-	      astr_cat_cstr (epath, pw->pw_dir);
-	    }
-	}
+        {
+          if (*(sp + 1) == '/')
+            {
+              /* Got `~/'. Restart from this point and insert the user's
+                 home directory. */
+              astr_truncate (epath, 0);
+              pw = getpwuid (getuid ());
+              if (pw == NULL)
+                {
+                  ret = false;
+                  break;
+                }
+              if (strcmp (pw->pw_dir, "/") != 0)
+                astr_cat_cstr (epath, pw->pw_dir);
+              ++sp;
+            }
+          else
+            {
+              /* Got `~something'.  Restart from this point and insert that
+                 user's home directory. */
+              astr as = astr_new ();
+              astr_truncate (epath, 0);
+              ++sp;
+              while (*sp != '\0' && *sp != '/')
+                astr_cat_char (as, *sp++);
+              pw = getpwnam (astr_cstr (as));
+              astr_delete (as);
+              if (pw == NULL)
+                {
+                  ret = false;
+                  break;
+                }
+              astr_cat_cstr (epath, pw->pw_dir);
+            }
+        }
       else if (*sp == '.')
-	{
-	  if (*(sp + 1) == '/' || *(sp + 1) == '\0')
-	    {
-	      ++sp;
-	      if (*sp == '/' && *(sp + 1) != '/')
-		++sp;
-	    }
-	  else if (*(sp + 1) == '.' &&
-		   (*(sp + 2) == '/' || *(sp + 2) == '\0'))
-	    {
-	      if (astr_len (epath) >= 1 && *astr_char (epath, -1) == '/')
-		astr_truncate (epath, -1);
-	      while (*astr_char (epath, -1) != '/' && astr_len (epath) >= 1)
-		astr_truncate (epath, -1);
-	      sp += 2;
-	      if (*sp == '/' && *(sp + 1) != '/')
-		++sp;
-	    }
-	  else
-	    goto got_component;
-	}
+        {
+          if (*(sp + 1) == '/' || *(sp + 1) == '\0')
+            {
+              ++sp;
+              if (*sp == '/' && *(sp + 1) != '/')
+                ++sp;
+            }
+          else if (*(sp + 1) == '.' &&
+                   (*(sp + 2) == '/' || *(sp + 2) == '\0'))
+            /* FIXME:
+             *
+             * If /foo on your system is a symlink to /bar/baz, then /foo/../quux
+             * is actually /bar/quux, not /quux as a naive ../-removal would give
+             * you. If you want to do this kind of processing, you probably want
+             * "Cwd"'s realpath function
+             */
+            {
+              if (astr_len (epath) >= 1 && *astr_char (epath, -1) == '/')
+                astr_truncate (epath, -1);
+              while (*astr_char (epath, -1) != '/' && astr_len (epath) >= 1)
+                astr_truncate (epath, -1);
+              sp += 2;
+              if (*sp == '/' && *(sp + 1) != '/')
+                ++sp;
+            }
+          else
+            goto got_component;
+        }
       else
-	{
-	  const char *p;
-	got_component:
-	  p = sp;
-	  while (*p != '\0' && *p != '/')
-	    p++;
-	  if (*p == '\0')
-	    {			/* Final filename */
-	      astr_cat_cstr (epath, sp);
-	      break;
-	    }
-	  else
-	    {			/* Non-final directory */
-	      while (*sp != '/')
-		astr_cat_char (epath, *sp++);
-	    }
-	}
+        {
+          const char *p;
+        got_component:
+          p = sp;
+          while (*p != '\0' && *p != '/')
+            p++;
+          if (*p == '\0')
+            {			/* Final filename */
+              astr_cat_cstr (epath, sp);
+              break;
+            }
+          else
+            {			/* Non-final directory */
+              while (*sp != '/')
+                astr_cat_char (epath, *sp++);
+            }
+        }
     }
 
   astr_cpy (path, epath);
@@ -222,11 +215,10 @@ expand_path (astr path)
 
 /*
  * Return a `~/foo' like path if the user is under his home directory,
- * and restart from / if // found,
- * else the unmodified path.
+ * and restart from / if // found, else the unmodified path.
  */
 astr
-compact_path (const astr path)
+compact_path (astr path)
 {
   astr buf = astr_new ();
   size_t i;
@@ -245,9 +237,9 @@ compact_path (const astr path)
     {
       astr_cpy_cstr (buf, "~/");
       if (!strcmp (pw->pw_dir, "/"))
-	astr_cat_cstr (buf, astr_char (path, 1));
+        astr_cat_cstr (buf, astr_char (path, 1));
       else
-	astr_cat_cstr (buf, astr_char (path, i + 1));
+        astr_cat_cstr (buf, astr_char (path, i + 1));
     }
   else
     astr_cpy (buf, path);
@@ -256,27 +248,28 @@ compact_path (const astr path)
 }
 
 /*
- * Return the current directory.
+ * Return the current directory for the buffer.
  */
 static astr
-get_current_dir (void)
+get_buffer_dir (void)
 {
   astr buf;
-  int p;
+  const char *p, *q;
 
   if (cur_bp->filename != NULL)
     /* If the current buffer has a filename, get the current directory
        name from it. */
     buf = astr_new_cstr (cur_bp->filename);
   else
-    {				/* Get the current directory name from the system. */
+    { /* Get the current directory name from the system. */
       buf = agetcwd ();
       if (astr_len (buf) != 0 && *astr_char (buf, -1) != '/')
-	astr_cat_char (buf, '/');
+        astr_cat_char (buf, '/');
     }
 
-  p = astr_rfind_cstr (buf, "/");
-  astr_truncate (buf, p + 1);
+  p = astr_cstr (buf);
+  q = strrchr (p, '/');
+  astr_truncate (buf, q ? q - p + 1 : 0);
 
   return buf;
 }
@@ -288,12 +281,9 @@ astr
 get_home_dir (void)
 {
   char *s = getenv ("HOME");
-  astr as;
-  if (s != NULL && strlen (s) < PATH_MAX)
-    as = astr_new_cstr (s);
-  else
-    as = agetcwd ();
-  return as;
+  if (s != NULL)
+    return astr_new_cstr (s);
+  return NULL;
 }
 
 /* Return nonzero if file exists and can be written. */
@@ -315,7 +305,7 @@ char coding_eol_cr[3] = "\r";
  * Read the file contents into a buffer.
  * Return quietly if the file doesn't exist, or other error.
  */
-void
+static void
 read_from_disk (const char *filename)
 {
   Line *lp;
@@ -328,10 +318,10 @@ read_from_disk (const char *filename)
   if (fp == NULL)
     {
       if (errno != ENOENT)
-	{
-	  minibuf_write ("%s: %s", filename, strerror (errno));
-	  cur_bp->flags |= BFLAG_READONLY;
-	}
+        {
+          minibuf_write ("%s: %s", filename, strerror (errno));
+          cur_bp->flags |= BFLAG_READONLY;
+        }
       return;
     }
 
@@ -345,58 +335,58 @@ read_from_disk (const char *filename)
   if (size > 0)
     {
       for (i = 0; i < size && total_eols < MAX_EOL_CHECK_COUNT; i++)
-	{
-	  if (buf[i] == '\n' || buf[i] == '\r')
-	    {
-	      total_eols++;
-	      if (buf[i] == '\n')
-		this_eol_type = coding_eol_lf;
-	      else if (i >= size || buf[i + 1] != '\n')
-		this_eol_type = coding_eol_cr;
-	      else
-		{
-		  this_eol_type = coding_eol_crlf;
-		  i++;
-		}
+        {
+          if (buf[i] == '\n' || buf[i] == '\r')
+            {
+              total_eols++;
+              if (buf[i] == '\n')
+                this_eol_type = coding_eol_lf;
+              else if (i >= size || buf[i + 1] != '\n')
+                this_eol_type = coding_eol_cr;
+              else
+                {
+                  this_eol_type = coding_eol_crlf;
+                  i++;
+                }
 
-	      if (first_eol)
-		{
-		  /* This is the first end-of-line. */
-		  cur_bp->eol = this_eol_type;
-		  first_eol = false;
-		}
-	      else if (cur_bp->eol != this_eol_type)
-		{
-		  /* This EOL is different from the last; arbitrarily choose
-		     LF. */
-		  cur_bp->eol = coding_eol_lf;
-		  break;
-		}
-	    }
-	}
+              if (first_eol)
+                {
+                  /* This is the first end-of-line. */
+                  cur_bp->eol = this_eol_type;
+                  first_eol = false;
+                }
+              else if (cur_bp->eol != this_eol_type)
+                {
+                  /* This EOL is different from the last; arbitrarily choose
+                     LF. */
+                  cur_bp->eol = coding_eol_lf;
+                  break;
+                }
+            }
+        }
 
       /* Process this and subsequent chunks into lines. */
       eol_len = strlen (cur_bp->eol);
       do
-	{
-	  for (i = 0; i < size; i++)
-	    {
-	      if (strncmp (cur_bp->eol, buf + i, eol_len) != 0)
-		astr_cat_char (lp->item, buf[i]);
-	      else
-		{
-		  lp = list_prepend (lp, astr_new ());
-		  ++cur_bp->num_lines;
-		  i += eol_len - 1;
-		}
-	    }
-	}
+        {
+          for (i = 0; i < size; i++)
+            {
+              if (strncmp (cur_bp->eol, buf + i, eol_len) != 0)
+                astr_cat_char (lp->text, buf[i]);
+              else
+                {
+                  lp = line_insert (lp, astr_new ());
+                  ++cur_bp->last_line;
+                  i += eol_len - 1;
+                }
+            }
+        }
       while ((size = fread (buf, 1, BUFSIZ, fp)) > 0);
     }
 
-  list_next (lp) = cur_bp->lines;
-  list_prev (cur_bp->lines) = lp;
-  cur_bp->pt.p = list_next (cur_bp->lines);
+  lp->next = cur_bp->lines;
+  cur_bp->lines->prev = lp;
+  cur_bp->pt.p = cur_bp->lines->next;
 
   fclose (fp);
 }
@@ -410,8 +400,8 @@ find_file (const char *filename)
   for (bp = head_bp; bp != NULL; bp = bp->next)
     if (bp->filename != NULL && !strcmp (bp->filename, filename))
       {
-	switch_to_buffer (bp);
-	return true;
+        switch_to_buffer (bp);
+        return true;
       }
 
   s = make_buffer_name (filename);
@@ -440,7 +430,7 @@ find_file (const char *filename)
   return true;
 }
 
-Completion *
+static Completion *
 make_buffer_completion (void)
 {
   Buffer *bp;
@@ -448,7 +438,8 @@ make_buffer_completion (void)
 
   cp = completion_new (false);
   for (bp = head_bp; bp != NULL; bp = bp->next)
-    list_append (cp->completions, xstrdup (bp->name));
+    gl_sortedlist_add (cp->completions, completion_strcmp,
+                       xstrdup (bp->name));
 
   return cp;
 }
@@ -460,25 +451,19 @@ Switch to a buffer visiting the file,
 creating one if none already exists.
 +*/
 {
-  astr buf = get_current_dir ();
-  char *ms = minibuf_read_dir ("Find file: ", astr_cstr (buf));
+  astr buf = get_buffer_dir ();
+  char *ms = minibuf_read_filename ("Find file: ", astr_cstr (buf), NULL);
+  int ret = false;
 
-  if (ms == NULL)
-    {
-      astr_delete (buf);
-      return cancel ();
-    }
   astr_delete (buf);
+  if (ms == NULL)
+    return FUNCALL (keyboard_quit);
 
   if (ms[0] != '\0')
-    {
-      int ret_value = find_file (ms);
-      free (ms);
-      return ret_value;
-    }
+    ret = find_file (ms);
 
   free (ms);
-  return false;
+  return bool_to_lisp (ret);
 }
 END_DEFUN
 
@@ -489,13 +474,42 @@ Like `find-file' but marks buffer as read-only.
 Use M-x toggle-read-only to permit editing.
 +*/
 {
-  int ret_value = FUNCALL (find_file);
-  if (ret_value)
+  le * ret = FUNCALL (find_file);
+  if (ret == leT)
     cur_bp->flags |= BFLAG_READONLY;
-  return ret_value;
+  return ret;
 }
 END_DEFUN
 
+/*
+ * Check if the buffer has been modified.  If so, asks the user if
+ * he/she wants to save the changes.  If the response is positive, return
+ * true, else false.
+ */
+static int
+check_modified_buffer (Buffer * bp)
+{
+  int ans;
+
+  if (bp->flags & BFLAG_MODIFIED && !(bp->flags & BFLAG_NOSAVE))
+    for (;;)
+      {
+        if ((ans =
+             minibuf_read_yesno
+             ("Buffer %s modified; kill anyway? (yes or no) ",
+              bp->name)) == -1)
+          {
+            FUNCALL (keyboard_quit);
+            return false;
+          }
+        else if (!ans)
+          return false;
+        break;
+      }
+
+  return true;
+}
+
 DEFUN ("find-alternate-file", find_alternate_file)
 /*+
 Find the file specified by the user, select its buffer, kill previous buffer.
@@ -503,27 +517,23 @@ If the current buffer now contains an empty file that you just visited
 (presumably by mistake), use this command to visit the file you really want.
 +*/
 {
-  astr buf = get_current_dir ();
-  char *ms = minibuf_read_dir ("Find alternate: ", astr_cstr (buf));
+  const char *buf = cur_bp->filename;
+  char *base = base_name (cur_bp->filename);
+  char *ms = minibuf_read_filename ("Find alternate: ", buf, base);
+  int ret = false;
 
   if (ms == NULL)
-    {
-      astr_delete (buf);
-      return cancel ();
-    }
-  astr_delete (buf);
+    return FUNCALL (keyboard_quit);
 
   if (ms[0] != '\0' && check_modified_buffer (cur_bp))
     {
-      int ret_value;
       kill_buffer (cur_bp);
-      ret_value = find_file (ms);
-      free (ms);
-      return ret_value;
+      ret = find_file (ms);
     }
 
   free (ms);
-  return false;
+  free (base);
+  return bool_to_lisp (ret);
 }
 END_DEFUN
 
@@ -540,53 +550,28 @@ Select to the user specified buffer in the current window.
 
   cp = make_buffer_completion ();
   ms = minibuf_read_completion ("Switch to buffer (default %s): ",
-				"", cp, NULL, swbuf->name);
+                                "", cp, NULL, swbuf->name);
   free_completion (cp);
   if (ms == NULL)
-    return cancel ();
+    return FUNCALL (keyboard_quit);
 
   if (ms[0] != '\0')
     {
       swbuf = find_buffer (ms, false);
       if (swbuf == NULL)
-	{
-	  swbuf = find_buffer (ms, true);
-	  swbuf->flags = BFLAG_NEEDNAME | BFLAG_NOSAVE;
-	}
+        {
+          swbuf = find_buffer (ms, true);
+          swbuf->flags = BFLAG_NEEDNAME | BFLAG_NOSAVE;
+        }
     }
 
   switch_to_buffer (swbuf);
+  free ((char *) ms);
 
-  return true;
+  return leT;
 }
 END_DEFUN
 
-/*
- * Check if the buffer has been modified.  If so, asks the user if
- * he/she wants to save the changes.  If the response is positive, return
- * true, else false.
- */
-  int
-check_modified_buffer (Buffer * bp)
-{
-  int ans;
-
-  if (bp->flags & BFLAG_MODIFIED && !(bp->flags & BFLAG_NOSAVE))
-    for (;;)
-      {
-	if ((ans =
-	     minibuf_read_yesno
-	     ("Buffer %s modified; kill anyway? (yes or no) ",
-	      bp->name)) == -1)
-	  return cancel ();
-	else if (!ans)
-	  return false;
-	break;
-      }
-
-  return true;
-}
-
 /*
  * Remove the specified buffer from the buffer list and deallocate
  * its space.  Avoid killing the sole buffers and creates the scratch
@@ -607,32 +592,32 @@ kill_buffer (Buffer * kill_bp)
       Window *wp;
       Buffer *new_bp = create_buffer (cur_bp->name);
       /* If this is the sole buffer available, then remove the contents
-	 and set the name to `*scratch*' if it is not already set. */
+         and set the name to `*scratch*' if it is not already set. */
       assert (cur_bp == kill_bp);
 
       free_buffer (cur_bp);
 
       /* Scan all the windows that display this buffer. */
       for (wp = head_wp; wp != NULL; wp = wp->next)
-	if (wp->bp == cur_bp)
-	  {
-	    wp->bp = new_bp;
-	    wp->topdelta = 0;
-	    wp->saved_pt = NULL;	/* It was freed. */
-	  }
+        if (wp->bp == cur_bp)
+          {
+            wp->bp = new_bp;
+            wp->topdelta = 0;
+            wp->saved_pt = NULL;	/* It was freed. */
+          }
 
       head_bp = cur_bp = new_bp;
       head_bp->next = NULL;
       cur_bp->flags = BFLAG_NOSAVE | BFLAG_NEEDNAME | BFLAG_TEMPORARY;
       if (strcmp (cur_bp->name, "*scratch*") != 0)
-	{
-	  set_buffer_name (cur_bp, "*scratch*");
-	  if (cur_bp->filename != NULL)
-	    {
-	      free (cur_bp->filename);
-	      cur_bp->filename = NULL;
-	    }
-	}
+        {
+          set_buffer_name (cur_bp, "*scratch*");
+          if (cur_bp->filename != NULL)
+            {
+              free (cur_bp->filename);
+              cur_bp->filename = NULL;
+            }
+        }
     }
   else
     {
@@ -641,23 +626,23 @@ kill_buffer (Buffer * kill_bp)
 
       /* Search for windows displaying the buffer to kill. */
       for (wp = head_wp; wp != NULL; wp = wp->next)
-	if (wp->bp == kill_bp)
-	  {
-	    wp->bp = next_bp;
-	    wp->topdelta = 0;
-	    wp->saved_pt = NULL;	/* The marker will be freed. */
-	  }
+        if (wp->bp == kill_bp)
+          {
+            wp->bp = next_bp;
+            wp->topdelta = 0;
+            wp->saved_pt = NULL;	/* The marker will be freed. */
+          }
 
       /* Remove the buffer from the buffer list. */
       cur_bp = next_bp;
       if (head_bp == kill_bp)
-	head_bp = head_bp->next;
+        head_bp = head_bp->next;
       for (bp = head_bp; bp->next != NULL; bp = bp->next)
-	if (bp->next == kill_bp)
-	  {
-	    bp->next = bp->next->next;
-	    break;
-	  }
+        if (bp->next == kill_bp)
+          {
+            bp->next = bp->next->next;
+            break;
+          }
 
       free_buffer (kill_bp);
 
@@ -676,18 +661,19 @@ Kill the current buffer or the user specified one.
 
   cp = make_buffer_completion ();
   ms = minibuf_read_completion ("Kill buffer (default %s): ",
-				"", cp, NULL, cur_bp->name);
+                                "", cp, NULL, cur_bp->name);
   if (ms == NULL)
-    return cancel ();
+    return FUNCALL (keyboard_quit);
   free_completion (cp);
   if (ms[0] != '\0')
     {
       bp = find_buffer (ms, false);
       if (bp == NULL)
-	{
-	  minibuf_error ("Buffer `%s' not found", ms);
-	  return false;
-	}
+        {
+          minibuf_error ("Buffer `%s' not found", ms);
+          free ((char *) ms);
+          return leNIL;
+        }
     }
   else
     bp = cur_bp;
@@ -695,28 +681,42 @@ Kill the current buffer or the user specified one.
   if (check_modified_buffer (bp))
     {
       kill_buffer (bp);
-      return true;
+      free ((char *) ms);
+      return leT;
     }
 
-  return false;
+  free ((char *) ms);
+  return leNIL;
 }
 END_DEFUN
 
+static size_t
+insert_lines (size_t n, size_t end, size_t last, Line *from_lp)
+{
+  for (; n < end; n++, from_lp = from_lp->next)
+    {
+      insert_astr (from_lp->text);
+      if (n < last)
+        insert_newline ();
+    }
+  return n;
+}
+
 static void
 insert_buffer (Buffer * bp)
 {
-  Line *lp;
-  size_t size = calculate_buffer_size (bp), i;
+  Line *old_next = bp->pt.p->next;
+  astr old_cur_line = astr_cpy (astr_new (), bp->pt.p->text);
+  size_t old_cur_n = bp->pt.n, old_lines = bp->last_line;
+  size_t size = calculate_buffer_size (bp);
 
-  undo_save (UNDO_REMOVE_BLOCK, cur_bp->pt, size, 0);
+  undo_save (UNDO_REPLACE_BLOCK, cur_bp->pt, 0, size);
   undo_nosave = true;
-  for (lp = list_next (bp->lines); lp != bp->lines; lp = list_next (lp))
-    {
-      for (i = 0; i < astr_len (lp->item); i++)
-	insert_char (*astr_char (lp->item, (ptrdiff_t) i));
-      if (list_next (lp) != bp->lines)
-	insert_newline ();
-    }
+  insert_lines (0, old_cur_n, old_lines, bp->lines->next);
+  insert_astr (old_cur_line);
+  if (old_cur_n < old_lines)
+    insert_newline ();
+  insert_lines (old_cur_n + 1, old_lines, old_lines, old_next);
   undo_nosave = false;
 }
 
@@ -731,37 +731,33 @@ Puts mark after the inserted text.
   Completion *cp;
 
   if (warn_if_readonly_buffer ())
-    return false;
+    return leNIL;
 
   swbuf = ((cur_bp->next != NULL) ? cur_bp->next : head_bp);
   cp = make_buffer_completion ();
   ms = minibuf_read_completion ("Insert buffer (default %s): ",
-				"", cp, NULL, swbuf->name);
+                                "", cp, NULL, swbuf->name);
   if (ms == NULL)
-    return cancel ();
+    return FUNCALL (keyboard_quit);
   free_completion (cp);
   if (ms[0] != '\0')
     {
       bp = find_buffer (ms, false);
       if (bp == NULL)
-	{
-	  minibuf_error ("Buffer `%s' not found", ms);
-	  return false;
-	}
+        {
+          minibuf_error ("Buffer `%s' not found", ms);
+          free ((char *) ms);
+          return leNIL;
+        }
     }
   else
     bp = swbuf;
 
-  if (bp == cur_bp)
-    {
-      minibuf_error ("Cannot insert the current buffer");
-      return false;
-    }
-
   insert_buffer (bp);
-  set_mark_command ();
+  set_mark_interactive ();
 
-  return true;
+  free ((char *) ms);
+  return leT;
 }
 END_DEFUN
 
@@ -769,7 +765,7 @@ static int
 insert_file (char *filename)
 {
   int fd;
-  size_t i, size;
+  size_t size;
   char buf[BUFSIZ];
 
   if (!exist_file (filename))
@@ -793,14 +789,10 @@ insert_file (char *filename)
     }
 
   lseek (fd, 0, SEEK_SET);
-  undo_save (UNDO_REMOVE_BLOCK, cur_bp->pt, size, 0);
+  undo_save (UNDO_REPLACE_BLOCK, cur_bp->pt, 0, size);
   undo_nosave = true;
   while ((size = read (fd, buf, BUFSIZ)) > 0)
-    for (i = 0; i < size; i++)
-      if (buf[i] != '\n')
-	insert_char (buf[i]);
-      else
-	insert_newline ();
+    insert_nstring (buf, size);
   undo_nosave = false;
   close (fd);
 
@@ -817,40 +809,40 @@ Set mark after the inserted text.
   astr buf;
 
   if (warn_if_readonly_buffer ())
-    return false;
+    return leNIL;
 
-  buf = get_current_dir ();
-  ms = minibuf_read_dir ("Insert file: ", astr_cstr (buf));
+  buf = get_buffer_dir ();
+  ms = minibuf_read_filename ("Insert file: ", astr_cstr (buf), NULL);
   if (ms == NULL)
     {
       astr_delete (buf);
-      return cancel ();
+      return FUNCALL (keyboard_quit);
     }
   astr_delete (buf);
 
   if (ms[0] == '\0')
     {
       free (ms);
-      return false;
+      return leNIL;
     }
 
   if (!insert_file (ms))
     {
       free (ms);
-      return false;
+      return leNIL;
     }
 
-  set_mark_command ();
+  set_mark_interactive ();
 
   free (ms);
-  return true;
+  return leT;
 }
 END_DEFUN
 
 /*
  * Copy a file.
  */
-  static int
+static int
 copy_file (const char *source, const char *dest)
 {
   char buf[BUFSIZ];
@@ -869,7 +861,7 @@ copy_file (const char *source, const char *dest)
   if (xasprintf (&tname, "%s_XXXXXXXXXX", dest) == -1)
     {
       minibuf_error ("Cannot allocate temporary file name `%s'",
-		     strerror (errno));
+                     strerror (errno));
       return false;
     }
 
@@ -887,27 +879,23 @@ copy_file (const char *source, const char *dest)
   while ((len = read (ifd, buf, sizeof buf)) > 0)
     if (write (ofd, buf, len) < 0)
       {
-	minibuf_error ("Unable to write to backup file `%s'", dest);
-	close (ifd);
-	close (ofd);
-	return false;
+        minibuf_error ("Unable to write to backup file `%s'", dest);
+        close (ifd);
+        close (ofd);
+        return false;
       }
 
   serrno = errno;
   stat_valid = fstat (ifd, &st) != -1;
 
-#if defined(HAVE_FCHMOD) || defined(HAVE_FCHOWN)
   /* Recover file permissions and ownership. */
   if (stat_valid)
     {
 #ifdef HAVE_FCHMOD
       fchmod (ofd, st.st_mode);
 #endif
-#ifdef HAVE_FCHOWN
       fchown (ofd, st.st_uid, st.st_gid);
-#endif
     }
-#endif
 
   close (ifd);
   close (ofd);
@@ -915,12 +903,12 @@ copy_file (const char *source, const char *dest)
   if (stat_valid)
     {
       if (rename (tname, dest) == -1)
-	{
-	  minibuf_error ("Cannot rename temporary file `%s'",
-			 strerror (errno));
-	  unlink (tname);
-	  stat_valid = false;
-	}
+        {
+          minibuf_error ("Cannot rename temporary file `%s'",
+                         strerror (errno));
+          unlink (tname);
+          stat_valid = false;
+        }
     }
   else if (unlink (tname) == -1)
     minibuf_error ("Cannot remove temporary file `%s'", strerror (errno));
@@ -955,17 +943,17 @@ raw_write_to_disk (Buffer * bp, const char *filename, int umask)
     return -1;
 
   /* Save the lines. */
-  for (lp = list_next (bp->lines); lp != bp->lines; lp = list_next (lp))
+  for (lp = bp->lines->next; lp != bp->lines; lp = lp->next)
     {
-      ssize_t len = (ssize_t) astr_len (lp->item);
+      ssize_t len = (ssize_t) astr_len (lp->text);
 
-      written = write (fd, astr_cstr (lp->item), len);
+      written = write (fd, astr_cstr (lp->text), len);
       if (written != len)
         {
           ret = written;
           break;
         }
-      if (list_next (lp) != bp->lines)
+      if (lp->next != bp->lines)
         {
           written = write (fd, bp->eol, eol_len);
           if (written != eol_len)
@@ -997,18 +985,18 @@ create_backup_filename (const char *filename, const char *backupdir)
 
       astr_cpy_cstr (buf, backupdir);
       if (*astr_char (buf, -1) != '/')
-	astr_cat_char (buf, '/');
+        astr_cat_char (buf, '/');
       while (*filename)
-	{
-	  if (*filename == '/')
-	    astr_cat_char (buf, '!');
-	  else
-	    astr_cat_char (buf, *filename);
-	  ++filename;
-	}
+        {
+          if (*filename == '/')
+            astr_cat_char (buf, '!');
+          else
+            astr_cat_char (buf, *filename);
+          ++filename;
+        }
 
       if (expand_path (buf) == NULL)
-	return NULL;
+        return NULL;
       res = buf;
     }
   else
@@ -1024,8 +1012,8 @@ create_backup_filename (const char *filename, const char *backupdir)
 static int
 write_to_disk (Buffer * bp, char *filename)
 {
-  int fd, backup = lookup_bool_variable ("make-backup-files"), ret;
-  char *backupdir = lookup_bool_variable ("backup-directory") ?
+  int fd, backup = get_variable_bool ("make-backup-files"), ret;
+  const char *backupdir = get_variable_bool ("backup-directory") ?
     get_variable ("backup-directory") : NULL;
 
   /*
@@ -1038,12 +1026,12 @@ write_to_disk (Buffer * bp, char *filename)
       close (fd);
       bfilename = create_backup_filename (filename, backupdir);
       if (bfilename && copy_file (filename, astr_cstr (bfilename)))
-	bp->flags |= BFLAG_BACKUP;
+        bp->flags |= BFLAG_BACKUP;
       else
-	{
-	  minibuf_error ("Cannot make backup file: %s", strerror (errno));
-	  waitkey (WAITKEY_DEFAULT);
-	}
+        {
+          minibuf_error ("Cannot make backup file: %s", strerror (errno));
+          waitkey (WAITKEY_DEFAULT);
+        }
       astr_delete (bfilename);
     }
 
@@ -1071,44 +1059,39 @@ save_buffer (Buffer * bp)
   else
     {
       if (bp->flags & BFLAG_NEEDNAME)
-	{
-	  ms = minibuf_read_dir ("File to save in: ", fname);
-	  if (ms == NULL)
-	    return cancel ();
-	  ms_is_from_minibuffer = 1;
-	  if (ms[0] == '\0')
-	    {
-	      free (ms);
-	      return false;
-	    }
+        {
+          ms = minibuf_read_filename ("File to save in: ", fname, NULL);
+          if (ms == NULL)
+            {
+              FUNCALL (keyboard_quit);
+              return false;
+            }
+          ms_is_from_minibuffer = 1;
+          if (ms[0] == '\0')
+            {
+              free (ms);
+              return false;
+            }
 
-	  set_buffer_filename (bp, ms);
+          set_buffer_filename (bp, ms);
 
-	  bp->flags &= ~BFLAG_NEEDNAME;
-	}
+          bp->flags &= ~BFLAG_NEEDNAME;
+        }
       else
-	ms = bp->filename;
+        ms = bp->filename;
 
       if (write_to_disk (bp, ms))
-	{
-	  Undo *up;
+        {
+          minibuf_write ("Wrote %s", ms);
+          bp->flags &= ~BFLAG_MODIFIED;
 
-	  minibuf_write ("Wrote %s", ms);
-	  bp->flags &= ~BFLAG_MODIFIED;
-
-	  /* Set unchanged flags to false except for the
-	     last undo action, which is set to true. */
-	  up = bp->last_undop;
-	  if (up)
-	    up->unchanged = true;
-	  for (up = up->next; up; up = up->next)
-	    up->unchanged = false;
-	}
+          undo_set_unchanged (bp->last_undop);
+        }
 
       bp->flags &= ~BFLAG_TEMPORARY;
 
       if (ms_is_from_minibuffer)
-	free (ms);
+        free (ms);
     }
 
   return true;
@@ -1120,7 +1103,7 @@ Save current buffer in visited file if modified. By default, makes the
 previous version into a backup file if this is the first save.
 +*/
 {
-  return save_buffer (cur_bp);
+  return bool_to_lisp (save_buffer (cur_bp));
 }
 END_DEFUN
 
@@ -1131,16 +1114,21 @@ Makes buffer visit that file, and marks it not modified.
 +*/
 {
   char *fname = cur_bp->filename != NULL ? cur_bp->filename : cur_bp->name;
-  char *ms = minibuf_read_dir ("Write file: ", fname);
+  char *ms = minibuf_read_filename ("Write file: ", fname, NULL);
+
 
   if (ms == NULL)
-    return cancel ();
+    return FUNCALL (keyboard_quit);
   if (ms[0] == '\0')
     {
       free (ms);
-      return false;
+      return leNIL;
     }
 
+  //  if (file_exists (ms)
+      //(or (y-or-n-p (format "File `%s' exists; overwrite? " filename))
+// (error "Canceled")))
+
   set_buffer_filename (cur_bp, ms);
 
   cur_bp->flags &= ~(BFLAG_NEEDNAME | BFLAG_TEMPORARY);
@@ -1152,7 +1140,7 @@ Makes buffer visit that file, and marks it not modified.
     }
 
   free (ms);
-  return true;
+  return leT;
 }
 END_DEFUN
 
@@ -1165,63 +1153,64 @@ save_some_buffers (void)
   for (bp = head_bp; bp != NULL; bp = bp->next)
     if (bp->flags & BFLAG_MODIFIED && !(bp->flags & BFLAG_NOSAVE))
       {
-	char *fname = bp->filename != NULL ? bp->filename : bp->name;
+        char *fname = bp->filename != NULL ? bp->filename : bp->name;
 
-	++i;
+        ++i;
 
-	if (noask)
-	  save_buffer (bp);
-	else
-	  {
-	    for (;;)
-	      {
-		minibuf_write ("Save file %s? (y, n, !, ., q) ", fname);
+        if (noask)
+          save_buffer (bp);
+        else
+          {
+            for (;;)
+              {
+                minibuf_write ("Save file %s? (y, n, !, ., q) ", fname);
 
-		c = getkey ();
-		switch (c)
-		  {
-		  case KBD_CANCEL:
-		  case KBD_RET:
-		  case ' ':
-		  case 'y':
-		  case 'n':
-		  case 'q':
-		  case '.':
-		  case '!':
-		    goto exitloop;
-		  }
+                c = getkey ();
+                switch (c)
+                  {
+                  case KBD_CANCEL:
+                  case KBD_RET:
+                  case ' ':
+                  case 'y':
+                  case 'n':
+                  case 'q':
+                  case '.':
+                  case '!':
+                    goto exitloop;
+                  }
 
-		minibuf_error ("Please answer y, n, !, . or q.");
-		waitkey (WAITKEY_DEFAULT);
-	      }
+                minibuf_error ("Please answer y, n, !, . or q.");
+                waitkey (WAITKEY_DEFAULT);
+              }
 
-	  exitloop:
-	    minibuf_clear ();
+          exitloop:
+            minibuf_clear ();
 
-	    switch (c)
-	      {
-	      case KBD_CANCEL:	/* C-g */
-		return cancel ();
-	      case 'q':
-		goto endoffunc;
-	      case '.':
-		save_buffer (bp);
-		++i;
-		return true;
-	      case '!':
-		noask = true;
-		/* FALLTHROUGH */
-	      case ' ':
-	      case 'y':
-		save_buffer (bp);
-		++i;
-		break;
-	      case 'n':
-	      case KBD_RET:
-	      case KBD_DEL:
-		break;
-	      }
-	  }
+            switch (c)
+              {
+              case KBD_CANCEL:	/* C-g */
+                FUNCALL (keyboard_quit);
+                return false;
+              case 'q':
+                goto endoffunc;
+              case '.':
+                save_buffer (bp);
+                ++i;
+                return true;
+              case '!':
+                noask = true;
+                /* FALLTHROUGH */
+              case ' ':
+              case 'y':
+                save_buffer (bp);
+                ++i;
+                break;
+              case 'n':
+              case KBD_RET:
+              case KBD_DEL:
+                break;
+              }
+          }
       }
 
 endoffunc:
@@ -1237,7 +1226,7 @@ DEFUN ("save-some-buffers", save_some_buffers)
 Save some modified file-visiting buffers.  Asks user about each one.
 +*/
 {
-  return save_some_buffers ();
+  return bool_to_lisp (save_some_buffers ());
 }
 END_DEFUN
 
@@ -1250,7 +1239,7 @@ Offer to save each buffer, then kill this Zile process.
   int ans, i = 0;
 
   if (!save_some_buffers ())
-    return false;
+    return leNIL;
 
   for (bp = head_bp; bp != NULL; bp = bp->next)
     if (bp->flags & BFLAG_MODIFIED && !(bp->flags & BFLAG_NEEDNAME))
@@ -1259,18 +1248,18 @@ Offer to save each buffer, then kill this Zile process.
   if (i > 0)
     for (;;)
       {
-	if ((ans =
-	     minibuf_read_yesno
-	     ("Modified buffers exist; exit anyway? (yes or no) ", "")) == -1)
-	  return cancel ();
-	else if (!ans)
-	  return false;
-	break;
+        if ((ans =
+             minibuf_read_yesno
+             ("Modified buffers exist; exit anyway? (yes or no) ", "")) == -1)
+          return FUNCALL (keyboard_quit);
+        else if (!ans)
+          return leNIL;
+        break;
       }
 
   thisflag |= FLAG_QUIT_ZILE;
 
-  return true;
+  return leT;
 }
 END_DEFUN
 
@@ -1289,16 +1278,16 @@ zile_exit (int doabort)
   for (bp = head_bp; bp != NULL; bp = bp->next)
     if (bp->flags & BFLAG_MODIFIED && !(bp->flags & BFLAG_NOSAVE))
       {
-	astr buf;
-	buf = astr_new ();
-	if (bp->filename != NULL)
-	  astr_cpy_cstr (buf, bp->filename);
-	else
-	  astr_cpy_cstr (buf, bp->name);
-	astr_cat_cstr (buf, ".ZILESAVE");
-	fprintf (stderr, "Saving %s...\r\n", astr_cstr (buf));
-	raw_write_to_disk (bp, astr_cstr (buf), 0600);
-	astr_delete (buf);
+        astr buf;
+        buf = astr_new ();
+        if (bp->filename != NULL)
+          astr_cpy_cstr (buf, bp->filename);
+        else
+          astr_cpy_cstr (buf, bp->name);
+        astr_cat_cstr (buf, ".ZILESAVE");
+        fprintf (stderr, "Saving %s...\r\n", astr_cstr (buf));
+        raw_write_to_disk (bp, astr_cstr (buf), 0600);
+        astr_delete (buf);
       }
 
   if (doabort)
@@ -1314,36 +1303,36 @@ directory.
 +*/
 {
   struct stat st;
-  astr buf = get_current_dir ();
-  char *ms = minibuf_read_dir ("Change default directory: ",
-			 astr_cstr (buf));
+  astr buf = get_buffer_dir ();
+  char *ms = minibuf_read_filename ("Change default directory: ",
+                                    astr_cstr (buf), NULL);
 
   if (ms == NULL)
     {
       astr_delete (buf);
-      return cancel ();
+      return FUNCALL (keyboard_quit);
     }
   astr_delete (buf);
 
   if (ms[0] != '\0')
     {
       if (stat (ms, &st) != 0 || !S_ISDIR (st.st_mode))
-	{
-	  minibuf_error ("`%s' is not a directory", ms);
-	  free (ms);
-	  return false;
-	}
+        {
+          minibuf_error ("`%s' is not a directory", ms);
+          free (ms);
+          return leNIL;
+        }
       if (chdir (ms) == -1)
-	{
-	  minibuf_write ("%s: %s", ms, strerror (errno));
-	  free (ms);
-	  return false;
-	}
+        {
+          minibuf_write ("%s: %s", ms, strerror (errno));
+          free (ms);
+          return leNIL;
+        }
       free (ms);
-      return true;
+      return leT;
     }
 
   free (ms);
-  return false;
+  return leNIL;
 }
 END_DEFUN
