@@ -23,6 +23,7 @@
 
 #include "config.h"
 
+#include <assert.h>
 #include <stdarg.h>
 #include <stdlib.h>
 #include <string.h>
@@ -57,45 +58,69 @@ kill_ring_push_nstring (char *s, size_t size)
   astr_ncat_cstr (kill_ring_text, s, size);
 }
 
+static bool
+copy_or_kill_region (bool kill, Region * r)
+{
+  char *p = copy_text_block (r->start.n, r->start.o, r->size);
+
+  if (!(lastflag & FLAG_DONE_KILL))
+    free_kill_ring ();
+  kill_ring_push_nstring (p, r->size);
+  free (p);
+
+  if (kill)
+    {
+      if (cur_bp->flags & BFLAG_READONLY)
+        minibuf_error ("Read only text copied to kill ring");
+      else
+        assert (delete_region (r));
+    }
+
+  thisflag |= FLAG_DONE_KILL;
+  deactivate_mark ();
+
+  return true;
+}
+
 static int
 kill_line (int literally)
 {
+  if (eobp ())
+    {
+      minibuf_error ("End of buffer");
+      return false;
+    }
+
   if (!eolp ())
     {
-      if (warn_if_readonly_buffer ())
-	return false;
+      Region r;
 
-      undo_save (UNDO_INSERT_BLOCK, cur_bp->pt,
-		 astr_len (cur_bp->pt.p->item) - cur_bp->pt.o, 0);
-      undo_nosave = true;
-      while (!eolp ())
-	{
-	  kill_ring_push (following_char ());
-	  FUNCALL (delete_char);
-	}
-      undo_nosave = false;
+      r.start = cur_bp->pt;
+      r.size = astr_len (cur_bp->pt.p->text) - cur_bp->pt.o;
 
-      thisflag |= FLAG_DONE_KILL;
+      if (!copy_or_kill_region (true, &r))
+        return false;
 
       if (!literally)
-	return true;
+        return true;
     }
 
-  if (list_next (cur_bp->pt.p) != cur_bp->lines)
+  if (!eobp ())
     {
       if (!FUNCALL (delete_char))
-	return false;
+        return false;
 
       kill_ring_push ('\n');
-
       thisflag |= FLAG_DONE_KILL;
-
-      return true;
     }
 
-  minibuf_error ("End of buffer");
+  return true;
+}
 
-  return false;
+static int
+kill_line_literally (void)
+{
+  return kill_line (true);
 }
 
 DEFUN ("kill-line", kill_line)
@@ -104,30 +129,32 @@ Kill the rest of the current line; if no nonblanks there, kill thru newline.
 With prefix argument, kill that many lines from point.
 +*/
 {
-  int uni, ret = true;
+  le * ret = leT;
 
   if (!(lastflag & FLAG_DONE_KILL))
     free_kill_ring ();
 
   if (!(lastflag & FLAG_SET_UNIARG))
-    kill_line (lookup_bool_variable ("kill-whole-line"));
+    kill_line (get_variable_bool ("kill-whole-line"));
   else
-    {
-      undo_save (UNDO_START_SEQUENCE, cur_bp->pt, 0, 0);
-      for (uni = 0; uni < uniarg; ++uni)
-	if (!kill_line (true))
-	  {
-	    ret = false;
-	    break;
-	  }
-      undo_save (UNDO_END_SEQUENCE, cur_bp->pt, 0, 0);
-    }
+    ret = execute_with_uniarg (true, uniarg, kill_line_literally, NULL);
 
   deactivate_mark ();
   return ret;
 }
 END_DEFUN
 
+static bool
+copy_or_kill_the_region (bool kill)
+{
+  Region r;
+
+  if (!calculate_the_region (&r))
+    return false;
+
+  return copy_or_kill_region (kill, &r);
+}
+
 DEFUN ("kill-region", kill_region)
 /*+
 Kill between point and mark.
@@ -141,53 +168,7 @@ the text killed this time appends to the text killed last time
 to make one entry in the kill ring.
 +*/
 {
-  Region r;
-
-  if (!(lastflag & FLAG_DONE_KILL))
-    free_kill_ring ();
-
-  if (warn_if_no_mark ())
-    return false;
-
-  calculate_the_region (&r);
-
-  if (cur_bp->flags & BFLAG_READONLY)
-    {
-      /*
-       * The buffer is readonly; save only in the kill buffer
-       * and complain.
-       */
-      char *p;
-
-      p = copy_text_block (r.start.n, r.start.o, r.size);
-      kill_ring_push_nstring (p, r.size);
-      free (p);
-
-      warn_if_readonly_buffer ();
-    }
-  else
-    {
-      size_t size = r.size;
-
-      if (cur_bp->pt.p != r.start.p || r.start.o != cur_bp->pt.o)
-	FUNCALL (exchange_point_and_mark);
-      undo_save (UNDO_INSERT_BLOCK, cur_bp->pt, size, 0);
-      undo_nosave = true;
-      while (size--)
-	{
-	  if (!eolp ())
-	    kill_ring_push (following_char ());
-	  else
-	    kill_ring_push ('\n');
-	  FUNCALL (delete_char);
-	}
-      undo_nosave = false;
-    }
-
-  thisflag |= FLAG_DONE_KILL;
-  deactivate_mark ();
-
-  return true;
+  return bool_to_lisp (copy_or_kill_the_region (true));
 }
 END_DEFUN
 
@@ -196,27 +177,30 @@ DEFUN ("copy-region-as-kill", copy_region_as_kill)
 Save the region as if killed, but don't kill it.
 +*/
 {
-  Region r;
-  char *p;
+  return bool_to_lisp (copy_or_kill_the_region (false));
+}
+END_DEFUN
 
+static le *
+kill (int uniarg, Function mark_func)
+{
   if (!(lastflag & FLAG_DONE_KILL))
     free_kill_ring ();
 
-  if (warn_if_no_mark ())
-    return false;
+  if (warn_if_readonly_buffer ())
+    return leNIL;
 
-  calculate_the_region (&r);
-
-  p = copy_text_block (r.start.n, r.start.o, r.size);
-  kill_ring_push_nstring (p, r.size);
-  free (p);
+  push_mark ();
+  undo_save (UNDO_START_SEQUENCE, cur_bp->pt, 0, 0);
+  mark_func (uniarg, NULL);
+  FUNCALL (kill_region);
+  undo_save (UNDO_END_SEQUENCE, cur_bp->pt, 0, 0);
+  pop_mark ();
 
   thisflag |= FLAG_DONE_KILL;
-  deactivate_mark ();
-
-  return true;
+  minibuf_write ("");		/* Erase "Set mark" message.  */
+  return leT;
 }
-END_DEFUN
 
 DEFUN ("kill-word", kill_word)
 /*+
@@ -224,24 +208,7 @@ Kill characters forward until encountering the end of a word.
 With argument, do this that many times.
 +*/
 {
-  if (!(lastflag & FLAG_DONE_KILL))
-    free_kill_ring ();
-
-  if (warn_if_readonly_buffer ())
-    return false;
-
-  push_mark ();
-  undo_save (UNDO_START_SEQUENCE, cur_bp->pt, 0, 0);
-  FUNCALL_ARG (mark_word, uniarg);
-  FUNCALL (kill_region);
-  undo_save (UNDO_END_SEQUENCE, cur_bp->pt, 0, 0);
-  pop_mark ();
-
-  thisflag |= FLAG_DONE_KILL;
-
-  minibuf_write ("");		/* Don't write "Set mark" message.  */
-
-  return true;
+  return kill (uniarg, F_mark_word);
 }
 END_DEFUN
 
@@ -251,7 +218,7 @@ Kill characters backward until encountering the end of a word.
 With argument, do this that many times.
 +*/
 {
-  return FUNCALL_ARG (kill_word, !uniarg ? -1 : -uniarg);
+  return kill (-uniarg, F_mark_word);
 }
 END_DEFUN
 
@@ -262,24 +229,7 @@ With ARG, kill that many sexps after the cursor.
 Negative arg -N means kill N sexps before the cursor.
 +*/
 {
-  if (!(lastflag & FLAG_DONE_KILL))
-    free_kill_ring ();
-
-  if (warn_if_readonly_buffer ())
-    return false;
-
-  push_mark ();
-  undo_save (UNDO_START_SEQUENCE, cur_bp->pt, 0, 0);
-  FUNCALL_ARG (mark_sexp, uniarg);
-  FUNCALL (kill_region);
-  undo_save (UNDO_END_SEQUENCE, cur_bp->pt, 0, 0);
-  pop_mark ();
-
-  thisflag |= FLAG_DONE_KILL;
-
-  minibuf_write ("");		/* Don't write "Set mark" message.  */
-
-  return true;
+  return kill (uniarg, F_mark_sexp);
 }
 END_DEFUN
 
@@ -293,21 +243,21 @@ killed OR yanked.  Put point at end, and set mark at beginning.
   if (kill_ring_text == NULL)
     {
       minibuf_error ("Kill ring is empty");
-      return false;
+      return leNIL;
     }
 
   if (warn_if_readonly_buffer ())
-    return false;
+    return leNIL;
 
-  set_mark_command ();
+  set_mark_interactive ();
 
-  undo_save (UNDO_REMOVE_BLOCK, cur_bp->pt, astr_len (kill_ring_text), 0);
+  undo_save (UNDO_REPLACE_BLOCK, cur_bp->pt, 0, astr_len (kill_ring_text));
   undo_nosave = true;
-  insert_nstring (astr_cstr (kill_ring_text), astr_len (kill_ring_text));
+  insert_astr (kill_ring_text);
   undo_nosave = false;
 
   deactivate_mark ();
 
-  return true;
+  return leT;
 }
 END_DEFUN
