@@ -35,24 +35,90 @@
 #include "extern.h"
 
 
-static void
-adjust_markers (Line * newlp, Line * oldlp, size_t pointo, int dir,
-		int offset)
+/*
+ * Circular doubly-linked lists
+ */
+
+/* Create an empty list, returning a pointer to the list */
+Line *
+line_new (void)
 {
-  Marker *pt = point_marker (), *marker;
+  Line *l = XZALLOC (Line);
 
-  for (marker = cur_bp->markers; marker != NULL; marker = marker->next)
-    if (marker->pt.p == oldlp &&
-	(dir == -1 || marker->pt.o >= pointo + dir + (offset < 0)))
+  l->next = l->prev = l;
+  l->text = NULL;
+
+  return l;
+}
+
+/* Delete a list, freeing its nodes */
+void
+line_delete (Line *l)
+{
+  while (l->next != l)
+    line_remove (l->next);
+
+  free (l);
+}
+
+/* Insert a line into list after the given point, returning the new line */
+Line *
+line_insert (Line *l, astr i)
+{
+  Line *n = XZALLOC (Line);
+
+  n->next = l->next;
+  n->prev = l;
+  n->text = i;
+  l->next = l->next->prev = n;
+
+  return n;
+}
+
+/* Remove a line from a list, if not sole line in list */
+void
+line_remove (Line *l)
+{
+  if (l->next || l->prev)
+    {
+      astr as = l->text;
+      l->prev->next = l->next;
+      l->prev->next->prev = l->prev;
+      free (l);
+      astr_delete (as);
+    }
+}
+
+
+/*
+ * Adjust markers (including point) when line at point is split, or
+ * next line is joined on, or where a line is edited.
+ *   newlp is the line to which characters were moved, oldlp the line
+ *    moved from (if dir == 0, newlp == oldlp)
+ *   pointo is point at which oldlp was split (to make newlp) or
+ *     joined to newlp
+ *   dir is 1 for split, -1 for join or 0 for line edit (newlp == oldlp)
+ *   if dir == 0, delta gives the number of characters inserted (>0) or
+ *     deleted (<0)
+ */
+static void
+adjust_markers (Line * newlp, Line * oldlp, size_t pointo, int dir, int delta)
+{
+  Marker *pt = point_marker (), *m;
+
+  assert (dir >= -1 && dir <= 1);
+
+  for (m = cur_bp->markers; m != NULL; m = m->next)
+    if (m->pt.p == oldlp && (dir == -1 || m->pt.o > pointo))
       {
-	marker->pt.p = newlp;
-	marker->pt.o -= pointo * dir - offset;
-	marker->pt.n += dir;
+        m->pt.p = newlp;
+        m->pt.o += delta - (pointo * dir);
+        m->pt.n += dir;
       }
-    else if (marker->pt.n > cur_bp->pt.n)
-      marker->pt.n += dir;
+    else if (m->pt.n > cur_bp->pt.n)
+      m->pt.n += dir;
 
-  cur_bp->pt = pt->pt;
+  cur_bp->pt = pt->pt; /* This marker has been updated to new position */
   free_marker (pt);
 }
 
@@ -60,14 +126,14 @@ adjust_markers (Line * newlp, Line * oldlp, size_t pointo, int dir,
  * whatever the insert/overwrite mode is.
  * This function doesn't change the current position of the pointer.
  */
-int
+static int
 intercalate_char (int c)
 {
   if (warn_if_readonly_buffer ())
     return false;
 
-  undo_save (UNDO_REMOVE_CHAR, cur_bp->pt, 0, 0);
-  astr_insert_char (cur_bp->pt.p->item, (ptrdiff_t) cur_bp->pt.o, c);
+  undo_save (UNDO_REPLACE_BLOCK, cur_bp->pt, 0, 1);
+  astr_insert_char (cur_bp->pt.p->text, (ptrdiff_t) cur_bp->pt.o, c);
   cur_bp->flags |= BFLAG_MODIFIED;
 
   return true;
@@ -88,27 +154,25 @@ insert_char (int c)
   if (cur_bp->flags & BFLAG_OVERWRITE)
     {
       /* Current character isn't the end of line
-	 && isn't a \t
-	 || tab width isn't correct
-	 || current char is a \t && we are in the tab limit.  */
-      if ((cur_bp->pt.o < astr_len (cur_bp->pt.p->item))
-	  &&
-	  ((*astr_char (cur_bp->pt.p->item, (ptrdiff_t) cur_bp->pt.o) != '\t')
-	   ||
-	   ((*astr_char (cur_bp->pt.p->item, (ptrdiff_t) cur_bp->pt.o) ==
-	     '\t') && ((get_goalc () % t) == t))))
-	{
-	  /* Replace the character.  */
-	  undo_save (UNDO_REPLACE_CHAR, cur_bp->pt,
-		     (size_t) (*astr_char (cur_bp->pt.p->item,
-					   (ptrdiff_t) cur_bp->pt.o)), 0);
-	  *astr_char (cur_bp->pt.p->item, (ptrdiff_t) cur_bp->pt.o) = c;
-	  ++cur_bp->pt.o;
+         && isn't a \t
+         || tab width isn't correct
+         || current char is a \t && we are in the tab limit.  */
+      if ((cur_bp->pt.o < astr_len (cur_bp->pt.p->text))
+          &&
+          ((*astr_char (cur_bp->pt.p->text, (ptrdiff_t) cur_bp->pt.o) != '\t')
+           ||
+           ((*astr_char (cur_bp->pt.p->text, (ptrdiff_t) cur_bp->pt.o) ==
+             '\t') && ((get_goalc () % t) == t))))
+        {
+          /* Replace the character.  */
+          undo_save (UNDO_REPLACE_BLOCK, cur_bp->pt, 1, 1);
+          *astr_char (cur_bp->pt.p->text, (ptrdiff_t) cur_bp->pt.o) = c;
+          ++cur_bp->pt.o;
 
-	  cur_bp->flags |= BFLAG_MODIFIED;
+          cur_bp->flags |= BFLAG_MODIFIED;
 
-	  return true;
-	}
+          return true;
+        }
       /*
        * Fall through the "insertion" mode of a character
        * at the end of the line, since it is totally
@@ -117,6 +181,7 @@ insert_char (int c)
     }
 
   intercalate_char (c);
+  forward_char ();
   adjust_markers (cur_bp->pt.p, cur_bp->pt.p, cur_bp->pt.o, 0, 1);
 
   return true;
@@ -152,14 +217,14 @@ insert_expanded_tab (int (*inschr) (int chr))
   undo_save (UNDO_END_SEQUENCE, cur_bp->pt, 0, 0);
 }
 
-int
+static int
 insert_tab (void)
 {
   if (warn_if_readonly_buffer ())
     return false;
 
-  if (lookup_bool_variable ("indent-tabs-mode"))
-    insert_char ('\t');
+  if (get_variable_bool ("indent-tabs-mode"))
+    insert_char_in_insert_mode ('\t');
   else
     insert_expanded_tab (insert_char_in_insert_mode);
 
@@ -172,57 +237,31 @@ Insert a tabulation at the current point position into the current
 buffer.
 +*/
 {
-  int uni, ret = true;
-
-  undo_save (UNDO_START_SEQUENCE, cur_bp->pt, 0, 0);
-  for (uni = 0; uni < uniarg; ++uni)
-    if (!insert_tab ())
-      {
-	ret = false;
-	break;
-      }
-  undo_save (UNDO_END_SEQUENCE, cur_bp->pt, 0, 0);
-
-  return ret;
+  return execute_with_uniarg (true, uniarg, insert_tab, NULL);
 }
 END_DEFUN
 
 /*
  * Insert a newline at the current position without moving the cursor.
- * Update all other cursors if they point on the splitted line.
+ * Update markers after point in the split line.
  */
-  int
-intercalate_newline ()
+static int
+intercalate_newline (void)
 {
-  Line *lp1, *lp2;
-  size_t lp1len, lp2len;
-  astr as;
-
   if (warn_if_readonly_buffer ())
     return false;
 
-  undo_save (UNDO_REMOVE_CHAR, cur_bp->pt, 0, 0);
+  undo_save (UNDO_REPLACE_BLOCK, cur_bp->pt, 0, 1);
 
-  /* Calculate the two line lengths. */
-  lp1len = cur_bp->pt.o;
-  lp2len = astr_len (cur_bp->pt.p->item) - lp1len;
-
-  lp1 = cur_bp->pt.p;
-
-  /* Update line linked list. */
-  lp2 = list_prepend (lp1, astr_new ());
-  ++cur_bp->num_lines;
-
-  /* Move the text after the point into the new line. */
-  as = astr_substr (lp1->item, (ptrdiff_t) lp1len, lp2len);
-  astr_cpy (lp2->item, as);
-  astr_delete (as);
-  astr_truncate (lp1->item, (ptrdiff_t) lp1len);
-
-  adjust_markers (lp2, lp1, lp1len, 1, 0);
+  /* Move the text after the point into a new line. */
+  line_insert (cur_bp->pt.p,
+               astr_substr (cur_bp->pt.p->text, (ptrdiff_t) cur_bp->pt.o,
+                            astr_len (cur_bp->pt.p->text) - cur_bp->pt.o));
+  ++cur_bp->last_line;
+  astr_truncate (cur_bp->pt.p->text, (ptrdiff_t) cur_bp->pt.o);
+  adjust_markers (cur_bp->pt.p->next, cur_bp->pt.p, cur_bp->pt.o, 1, 0);
 
   cur_bp->flags |= BFLAG_MODIFIED;
-
   thisflag |= FLAG_NEED_RESYNC;
 
   return true;
@@ -277,30 +316,20 @@ recase (char *str, size_t len, const char *tmpl, size_t tmpl_len)
  */
 void
 line_replace_text (Line ** lp, size_t offset, size_t oldlen,
-		   char *newtext, size_t newlen, int replace_case)
+                   char *newtext, size_t newlen, int replace_case)
 {
-  if (oldlen == 0)
-    return;
+  replace_case = replace_case && get_variable_bool ("case-replace");
 
-  if (replace_case && lookup_bool_variable ("case-replace"))
+  if (replace_case)
     {
       newtext = xstrdup (newtext);
-      recase (newtext, newlen, astr_char ((*lp)->item, (ptrdiff_t) offset),
-	      oldlen);
+      recase (newtext, newlen, astr_char ((*lp)->text, (ptrdiff_t) offset),
+              oldlen);
     }
 
-  if (newlen != oldlen)
-    {
-      cur_bp->flags |= BFLAG_MODIFIED;
-      astr_replace_cstr ((*lp)->item, (ptrdiff_t) offset, oldlen, newtext);
-      adjust_markers (*lp, *lp, offset, 0, (int) (newlen - oldlen));
-    }
-  else if (memcmp (astr_char ((*lp)->item, (ptrdiff_t) offset),
-		   newtext, newlen) != 0)
-    {
-      memcpy (astr_char ((*lp)->item, (ptrdiff_t) offset), newtext, newlen);
-      cur_bp->flags |= BFLAG_MODIFIED;
-    }
+  cur_bp->flags |= BFLAG_MODIFIED;
+  astr_replace_cstr ((*lp)->text, (ptrdiff_t) offset, oldlen, newtext);
+  adjust_markers (*lp, *lp, offset, 0, (int) (newlen - oldlen));
 
   if (replace_case)
     free ((char *) newtext);
@@ -311,8 +340,10 @@ line_replace_text (Line ** lp, size_t offset, size_t oldlen,
  * right-most space character at or before fill-column, if there is
  * one, or at the left-most at or after fill-column, if not. If the
  * line contains no spaces, no break is made.
+ *
+ * Return flag indicating whether break was made.
  */
-void
+bool
 fill_break_line (void)
 {
   size_t i, break_col = 0, excess = 0, old_col;
@@ -320,7 +351,7 @@ fill_break_line (void)
 
   /* If we're not beyond fill-column, stop now. */
   if (get_goalc () <= fillcol)
-    return;
+    return false;
 
   /* Move cursor back to fill column */
   old_col = cur_bp->pt.o;
@@ -333,41 +364,52 @@ fill_break_line (void)
   /* Find break point moving left from fill-column. */
   for (i = cur_bp->pt.o; i > 0; i--)
     {
-      int c = *astr_char (cur_bp->pt.p->item, (ptrdiff_t) (i - 1));
+      int c = *astr_char (cur_bp->pt.p->text, (ptrdiff_t) (i - 1));
       if (isspace (c))
-	{
-	  break_col = i;
-	  break;
-	}
+        {
+          break_col = i;
+          break;
+        }
     }
 
   /* If no break point moving left from fill-column, find first
      possible moving right. */
   if (break_col == 0)
     {
-      for (i = cur_bp->pt.o + 1; i < astr_len (cur_bp->pt.p->item); i++)
-	{
-	  int c = *astr_char (cur_bp->pt.p->item, (ptrdiff_t) (i - 1));
-	  if (isspace (c))
-	    {
-	      break_col = i;
-	      break;
-	    }
-	}
+      for (i = cur_bp->pt.o + 1; i < astr_len (cur_bp->pt.p->text); i++)
+        {
+          int c = *astr_char (cur_bp->pt.p->text, (ptrdiff_t) (i - 1));
+          if (isspace (c))
+            {
+              break_col = i;
+              break;
+            }
+        }
     }
 
-  if (break_col >= 1)
+  if (break_col >= 1) /* Break line. */
     {
-      /* Break line. */
       size_t last_col = cur_bp->pt.o - break_col;
       cur_bp->pt.o = break_col;
       FUNCALL (delete_horizontal_space);
       insert_newline ();
       cur_bp->pt.o = last_col + excess;
+      return true;
     }
-  else
-    /* Undo fiddling with point. */
-    cur_bp->pt.o = old_col;
+  else /* Undo fiddling with point. */
+    {
+      cur_bp->pt.o = old_col;
+      return false;
+    }
+}
+
+static int
+newline (void)
+{
+  if (cur_bp->flags & BFLAG_AUTOFILL &&
+      get_goalc () > (size_t) get_variable_number ("fill-column"))
+    fill_break_line ();
+  return insert_newline ();
 }
 
 DEFUN ("newline", newline)
@@ -376,23 +418,7 @@ Insert a newline at the current point position into
 the current buffer.
 +*/
 {
-  int uni, ret = true;
-
-  undo_save (UNDO_START_SEQUENCE, cur_bp->pt, 0, 0);
-  for (uni = 0; uni < uniarg; ++uni)
-    {
-      if (cur_bp->flags & BFLAG_AUTOFILL &&
-	  get_goalc () > (size_t) get_variable_number ("fill-column"))
-	fill_break_line ();
-      if (!insert_newline ())
-	{
-	  ret = false;
-	  break;
-	}
-    }
-  undo_save (UNDO_END_SEQUENCE, cur_bp->pt, 0, 0);
-
-  return ret;
+  return execute_with_uniarg (true, uniarg, newline, NULL);
 }
 END_DEFUN
 
@@ -401,45 +427,30 @@ DEFUN ("open-line", open_line)
 Insert a newline and leave point before it.
 +*/
 {
-  int uni, ret = true;
-
-  undo_save (UNDO_START_SEQUENCE, cur_bp->pt, 0, 0);
-  for (uni = 0; uni < uniarg; ++uni)
-    if (!intercalate_newline ())
-      {
-	ret = false;
-	break;
-      }
-  undo_save (UNDO_END_SEQUENCE, cur_bp->pt, 0, 0);
-
-  return ret;
+  return execute_with_uniarg (true, uniarg, intercalate_newline, NULL);
 }
 END_DEFUN
 
 void
-insert_string (const char *s)
+insert_nstring (const char *s, size_t len)
 {
-  undo_save (UNDO_REMOVE_BLOCK, cur_bp->pt, strlen (s), 0);
+  size_t i;
+  undo_save (UNDO_REPLACE_BLOCK, cur_bp->pt, 0, len);
   undo_nosave = true;
-  for (; *s != '\0'; ++s)
-    if (*s == '\n')
-      insert_newline ();
-    else
-      insert_char (*s);
+  for (i = 0; i < len; i++)
+    {
+      if (s[i] == '\n')
+        insert_newline ();
+      else
+        insert_char_in_insert_mode (s[i]);
+    }
   undo_nosave = false;
 }
 
 void
-insert_nstring (const char *s, size_t size)
+insert_astr (astr as)
 {
-  undo_save (UNDO_REMOVE_BLOCK, cur_bp->pt, size, 0);
-  undo_nosave = true;
-  for (; 0 < size--; ++s)
-    if (*s == '\n')
-      insert_newline ();
-    else
-      insert_char_in_insert_mode (*s);
-  undo_nosave = false;
+  insert_nstring (astr_cstr (as), astr_len (as));
 }
 
 void
@@ -447,11 +458,11 @@ bprintf (const char *fmt, ...)
 {
   va_list ap;
   char *buf;
+
   va_start (ap, fmt);
   xvasprintf (&buf, fmt, ap);
   va_end (ap);
-  insert_string (buf);
-  free (buf);
+  insert_nstring (buf, strlen (buf));
 }
 
 int
@@ -459,52 +470,77 @@ delete_char (void)
 {
   deactivate_mark ();
 
-  if (!eolp ())
+  if (eobp ())
     {
-      if (warn_if_readonly_buffer ())
-	return false;
-
-      undo_save (UNDO_INTERCALATE_CHAR, cur_bp->pt,
-		 (size_t) (*astr_char (cur_bp->pt.p->item,
-				       (ptrdiff_t) cur_bp->pt.o)), 0);
-      astr_remove (cur_bp->pt.p->item, (ptrdiff_t) cur_bp->pt.o, 1);
-      adjust_markers (cur_bp->pt.p, cur_bp->pt.p, cur_bp->pt.o, 0, -1);
-      cur_bp->flags |= BFLAG_MODIFIED;
-
-      return true;
+      minibuf_error ("End of buffer");
+      return false;
     }
-  else if (!eobp ())
+
+  if (warn_if_readonly_buffer ())
+    return false;
+
+  undo_save (UNDO_REPLACE_BLOCK, cur_bp->pt, 1, 0);
+
+  if (eolp ())
     {
-      Line *lp1, *lp2;
-      size_t lp1len;
+      size_t oldlen = astr_len (cur_bp->pt.p->text);
+      Line *oldlp = cur_bp->pt.p->next;
 
-      if (warn_if_readonly_buffer ())
-	return false;
-
-      undo_save (UNDO_INTERCALATE_CHAR, cur_bp->pt, '\n', 0);
-
-      lp1 = cur_bp->pt.p;
-      lp2 = list_next (lp1);
-      lp1len = astr_len (lp1->item);
-
-      /* Move the next line text into the current line. */
-      lp2 = list_next (cur_bp->pt.p);
-      astr_cat (lp1->item, list_next (cur_bp->pt.p)->item);
-      astr_delete (list_behead (lp1));
-      --cur_bp->num_lines;
-
-      adjust_markers (lp1, lp2, lp1len, -1, 0);
-
-      cur_bp->flags |= BFLAG_MODIFIED;
+      /* Join the lines. */
+      astr_cat (cur_bp->pt.p->text, oldlp->text);
+      line_remove (oldlp);
 
+      adjust_markers (cur_bp->pt.p, oldlp, oldlen, -1, 0);
+      --cur_bp->last_line;
       thisflag |= FLAG_NEED_RESYNC;
-
-      return true;
+    }
+  else
+    {
+      astr_remove (cur_bp->pt.p->text, (ptrdiff_t) cur_bp->pt.o, 1);
+      adjust_markers (cur_bp->pt.p, cur_bp->pt.p, cur_bp->pt.o, 0, -1);
     }
 
-  minibuf_error ("End of buffer");
+  cur_bp->flags |= BFLAG_MODIFIED;
 
-  return false;
+  return true;
+}
+
+static int
+backward_delete_char (void)
+{
+  deactivate_mark ();
+
+  if (!backward_char ())
+    {
+      minibuf_error ("Beginning of buffer");
+      return false;
+    }
+
+  delete_char ();
+  return true;
+}
+
+static int
+backward_delete_char_overwrite (void)
+{
+  if (bolp () || eolp ())
+    return backward_delete_char ();
+
+  deactivate_mark ();
+
+  if (warn_if_readonly_buffer ())
+    return false;
+
+  backward_char ();
+  if (following_char () == '\t')
+    insert_expanded_tab (insert_char);
+  else
+    insert_char (' ');
+  backward_char ();
+
+  cur_bp->flags |= BFLAG_MODIFIED;
+
+  return true;
 }
 
 DEFUN ("delete-char", delete_char)
@@ -513,87 +549,19 @@ Delete the following character.
 Join lines if the character is a newline.
 +*/
 {
-  int uni, ret = true;
-
-  if (uniarg < 0)
-    return FUNCALL_ARG (backward_delete_char, -uniarg);
-
-  undo_save (UNDO_START_SEQUENCE, cur_bp->pt, 0, 0);
-  for (uni = 0; uni < uniarg; ++uni)
-    if (!delete_char ())
-      {
-	ret = false;
-	break;
-      }
-  undo_save (UNDO_END_SEQUENCE, cur_bp->pt, 0, 0);
-
-  return ret;
+  return execute_with_uniarg (true, uniarg, delete_char, backward_delete_char);
 }
 END_DEFUN
 
-int
-backward_delete_char (void)
-{
-  deactivate_mark ();
-
-  if (backward_char ())
-    {
-      delete_char ();
-      return true;
-    }
-  else
-    {
-      minibuf_error ("Beginning of buffer");
-      return false;
-    }
-}
-
-static int
-backward_delete_char_overwrite (void)
-{
-  if (!bolp () && !eolp ())
-    {
-      if (warn_if_readonly_buffer ())
-	return false;
-
-      backward_char ();
-      if (following_char () == '\t')
-	insert_expanded_tab (insert_char);
-      else
-	insert_char (' ');
-      backward_char ();
-
-      cur_bp->flags |= BFLAG_MODIFIED;
-
-      return true;
-    }
-  else
-    return backward_delete_char ();
-}
-
 DEFUN ("backward-delete-char", backward_delete_char)
 /*+
 Delete the previous character.
 Join lines if the character is a newline.
 +*/
 {
-  int (*f) (void) = cur_bp->flags & BFLAG_OVERWRITE ?
+  int (*forward) (void) = cur_bp->flags & BFLAG_OVERWRITE ?
     backward_delete_char_overwrite : backward_delete_char;
-  int uni, ret = true;
-
-  if (uniarg < 0)
-    return FUNCALL_ARG (delete_char, -uniarg);
-
-  undo_save (UNDO_START_SEQUENCE, cur_bp->pt, 0, 0);
-  for (uni = 0; uni < uniarg; ++uni)
-    if (!(*f) ())
-      {
-	ret = false;
-	break;
-      }
-  undo_save (UNDO_END_SEQUENCE, cur_bp->pt, 0, 0);
-
-  return ret;
+  return execute_with_uniarg (true, uniarg, forward, delete_char);
 }
 END_DEFUN
 
@@ -611,7 +579,7 @@ Delete all spaces and tabs around point.
     backward_delete_char ();
 
   undo_save (UNDO_END_SEQUENCE, cur_bp->pt, 0, 0);
-  return true;
+  return leT;
 }
 END_DEFUN
 
@@ -624,23 +592,23 @@ Delete all spaces and tabs around point, leaving one space.
   FUNCALL (delete_horizontal_space);
   insert_char_in_insert_mode (' ');
   undo_save (UNDO_END_SEQUENCE, cur_bp->pt, 0, 0);
-  return true;
+  return leT;
 }
 END_DEFUN
 
 /***********************************************************************
-			 Indentation command
+                         Indentation command
 ***********************************************************************/
 /*
- * Go to cur_goalc() in the previous non-blank line.
+ * Go to cur_goalc () in the previous non-blank line.
  */
-  static void
+static void
 previous_nonblank_goalc (void)
 {
   size_t cur_goalc = get_goalc ();
 
   /* Find previous non-blank line. */
-  while (FUNCALL_ARG (forward_line, -1) && is_blank_line ());
+  while (FUNCALL_ARG (forward_line, -1) == leT && is_blank_line ());
 
   /* Go to `cur_goalc' in that non-blank line. */
   while (!eolp () && get_goalc () < cur_goalc)
@@ -661,16 +629,16 @@ does nothing.
 {
   size_t target_goalc = 0, cur_goalc = get_goalc ();
   Marker *old_point;
-  int ok = true;
+  int ret = true;
   size_t t = tab_width (cur_bp);
 
   if (warn_if_readonly_buffer ())
-    return false;
+    return leNIL;
 
   deactivate_mark ();
 
   /* If we're on first line, set target to 0. */
-  if (list_prev (cur_bp->pt.p) == cur_bp->lines)
+  if (cur_bp->pt.p->prev == cur_bp->lines)
     target_goalc = 0;
   else
     {				/* Find goalc in previous non-blank line. */
@@ -680,16 +648,16 @@ does nothing.
 
       /* Now find the next blank char. */
       if (!(preceding_char () == '\t' && get_goalc () > cur_goalc))
-	while (!eolp () && (!isspace (following_char ())))
-	  forward_char ();
+        while (!eolp () && (!isspace (following_char ())))
+          forward_char ();
 
       /* Find next non-blank char. */
       while (!eolp () && (isspace (following_char ())))
-	forward_char ();
+        forward_char ();
 
       /* Target column. */
       if (!eolp ())
-	target_goalc = get_goalc ();
+        target_goalc = get_goalc ();
 
       cur_bp->pt = old_point->pt;
       free_marker (old_point);
@@ -700,28 +668,28 @@ does nothing.
   if (target_goalc > 0)
     {
       /* If not at EOL on target line, insert spaces & tabs up to
-	 target_goalc; if already at EOL on target line, insert a tab.
+         target_goalc; if already at EOL on target line, insert a tab.
        */
       cur_goalc = get_goalc ();
       if (cur_goalc < target_goalc)
-	{
-	  do
-	    {
-	      if (cur_goalc % t == 0 && cur_goalc + t <= target_goalc)
-		ok = insert_tab ();
-	      else
-		ok = insert_char (' ');
-	    }
-	  while (ok && (cur_goalc = get_goalc ()) < target_goalc);
-	}
+        {
+          do
+            {
+              if (cur_goalc % t == 0 && cur_goalc + t <= target_goalc)
+                ret = insert_tab ();
+              else
+                ret = insert_char_in_insert_mode (' ');
+            }
+          while (ret && (cur_goalc = get_goalc ()) < target_goalc);
+        }
       else
-	ok = insert_tab ();
+        ret = insert_tab ();
     }
   else
-    ok = insert_tab ();
+    ret = insert_tab ();
   undo_save (UNDO_END_SEQUENCE, cur_bp->pt, 0, 0);
 
-  return ok;
+  return bool_to_lisp (ret);
 }
 END_DEFUN
 
@@ -755,11 +723,11 @@ If initial point was within line's indentation, position after
 the indentation.  Else stay at same point in text.
 +*/
 {
-  if (lookup_bool_variable ("tab-always-indent"))
-    return insert_tab ();
+  if (get_variable_bool ("tab-always-indent"))
+    return bool_to_lisp (insert_tab ());
   else if (get_goalc () < previous_line_indent ())
     return FUNCALL (indent_relative);
-  return true;
+  return leT;
 }
 END_DEFUN
 
@@ -769,39 +737,34 @@ Insert a newline, then indent.
 Indentation is done using the `indent-for-tab-command' function.
 +*/
 {
-  int ret;
+  le *ret = leNIL;
 
   if (warn_if_readonly_buffer ())
-    return false;
-  else
+    return leNIL;
+
+  deactivate_mark ();
+
+  undo_save (UNDO_START_SEQUENCE, cur_bp->pt, 0, 0);
+  if (insert_newline ())
     {
+      Marker *old_point = point_marker ();
       int indent;
+      size_t pos;
 
-      deactivate_mark ();
-
-      ret = insert_newline ();
-      if (ret)
-	{
-	  size_t pos;
-	  Marker *old_point = point_marker ();
-
-	  undo_save (UNDO_START_SEQUENCE, cur_bp->pt, 0, 0);
-
-	  /* Check where last non-blank goalc is. */
-	  previous_nonblank_goalc ();
-	  pos = get_goalc ();
-	  indent = pos > 0 || (!eolp () && isspace (following_char ()));
-	  cur_bp->pt = old_point->pt;
-	  free_marker (old_point);
-	  /* Only indent if we're in column > 0 or we're in column 0 and
-	     there is a space character there in the last non-blank line. */
-	  if (indent)
-	    FUNCALL (indent_for_tab_command);
-
-	  undo_save (UNDO_END_SEQUENCE, cur_bp->pt, 0, 0);
-	}
-
-      return ret;
+      /* Check where last non-blank goalc is. */
+      previous_nonblank_goalc ();
+      pos = get_goalc ();
+      indent = pos > 0 || (!eolp () && isspace (following_char ()));
+      cur_bp->pt = old_point->pt;
+      free_marker (old_point);
+      /* Only indent if we're in column > 0 or we're in column 0 and
+         there is a space character there in the last non-blank line. */
+      if (indent)
+        FUNCALL (indent_for_tab_command);
+      ret = leT;
     }
+  undo_save (UNDO_END_SEQUENCE, cur_bp->pt, 0, 0);
+
+  return ret;
 }
 END_DEFUN
