@@ -24,19 +24,15 @@
 #ifndef LISTS_H
 #define LISTS_H
 
-#include "astr.h"
-
 struct le
 {
   /* either data or a branch */
   struct le *branch;
   char *data;
   int quoted;
-  int tag;
 
   /* for the next in the list in the current parenlevel */
-  struct le *list_prev;
-  struct le *list_next;
+  struct le *next;
 };
 typedef struct le le;
 
@@ -44,19 +40,8 @@ le *leNew (const char *text);
 void leReallyWipe (le * list);
 void leWipe (le * list);
 
-le *leAddHead (le * list, le * element);
-le *leAddTail (le * list, le * element);
-
 le *leAddBranchElement (le * list, le * branch, int quoted);
 le *leAddDataElement (le * list, const char *data, int quoted);
 le *leDup (le * list);
 
-void leClearTag (le * list);
-void leTagData (le * list, char *data, int tagval);
-void leTagReplace (le * list, int tagval, le * newinfo);
-
-astr leDump (le * list, int indent);
-astr leDumpEval (le * list, int indent);
-astr leDumpReformat (le * tree);
-
 #endif
