@@ -33,6 +33,14 @@
 #include "extern.h"
 
 
+struct Macro
+{
+  size_t nkeys;		/* The number of keystrokes. */
+  size_t *keys;		/* Vector of keystrokes. */
+  char *name;		/* Name of the macro. */
+  Macro *next;		/* Next macro in the list. */
+};
+
 static Macro *cur_mp, *cmd_mp = NULL, *head_mp = NULL;
 
 static void
@@ -67,7 +75,7 @@ void
 add_key_to_cmd (size_t key)
 {
   if (cmd_mp == NULL)
-    cmd_mp = xzalloc (sizeof (Macro));
+    cmd_mp = XZALLOC (Macro);
 
   add_macro_key (cmd_mp, key);
 }
@@ -92,7 +100,7 @@ Use M-x name-last-kbd-macro to give it a permanent name.
   if (thisflag & FLAG_DEFINING_MACRO)
     {
       minibuf_error ("Already defining a keyboard macro");
-      return false;
+      return leNIL;
     }
 
   if (cur_mp)
@@ -101,8 +109,8 @@ Use M-x name-last-kbd-macro to give it a permanent name.
   minibuf_write ("Defining keyboard macro...");
 
   thisflag |= FLAG_DEFINING_MACRO;
-  cur_mp = xzalloc (sizeof (Macro));
-  return true;
+  cur_mp = XZALLOC (Macro);
+  return leT;
 }
 END_DEFUN
 
@@ -116,11 +124,11 @@ The macro is now available for use via C-x e.
   if (!(thisflag & FLAG_DEFINING_MACRO))
     {
       minibuf_error ("Not defining a keyboard macro");
-      return false;
+      return leNIL;
     }
 
   thisflag &= ~FLAG_DEFINING_MACRO;
-  return true;
+  return leT;
 }
 END_DEFUN
 
@@ -129,7 +137,7 @@ DEFUN ("name-last-kbd-macro", name_last_kbd_macro)
 Assign a name to the last keyboard macro defined.
 Argument SYMBOL is the name to define.
 The symbol's function definition becomes the keyboard macro string.
-Such a "function" cannot be called from Lisp, but it is a valid editor command.
+Such a \"function\" cannot be called from Lisp, but it is a valid editor command.
 +*/
 {
   Macro *mp;
@@ -139,25 +147,23 @@ Such a "function" cannot be called from Lisp, but it is a valid editor command.
   if (ms == NULL)
     {
       minibuf_error ("No command name given");
-      return false;
+      return leNIL;
     }
 
   if (cur_mp == NULL)
     {
       minibuf_error ("No keyboard macro defined");
-      return false;
+      return leNIL;
     }
 
   mp = get_macro (ms);
   if (mp)
-    {
-      /* If a macro with this name already exists, update its key list */
-      free (mp->keys);
-    }
+    /* If a macro with this name already exists, update its key list */
+    free (mp->keys);
   else
     {
       /* Add a new macro to the list */
-      mp = xzalloc (sizeof (*mp));
+      mp = XZALLOC (Macro);
       mp->next = head_mp;
       mp->name = xstrdup (ms);
       head_mp = mp;
@@ -169,7 +175,7 @@ Such a "function" cannot be called from Lisp, but it is a valid editor command.
   mp->keys = xzalloc (size);
   memcpy (mp->keys, cur_mp->keys, size);
 
-  return true;
+  return leT;
 }
 END_DEFUN
 
@@ -178,9 +184,7 @@ call_macro (Macro * mp)
 {
   size_t i;
 
-  /* The loop termination condition is really i >= 0, but unsigned
-     types are always >= 0, and we can't easily get SIZE_T_MAX. */
-  for (i = mp->nkeys - 1; i < mp->nkeys; i--)
+  for (i = mp->nkeys - 1; i != SIZE_MAX; i--)
     ungetkey (mp->keys[i]);
 }
 
@@ -193,12 +197,13 @@ To make a macro permanent so you can call it even after
 defining others, use M-x name-last-kbd-macro.
 +*/
 {
-  int uni, ret = true;
+  int uni;
+  le *ret = leT;
 
   if (cur_mp == NULL)
     {
       minibuf_error ("No kbd macro has been defined");
-      return false;
+      return leNIL;
     }
 
   undo_save (UNDO_START_SEQUENCE, cur_bp->pt, 0, 0);
@@ -213,7 +218,7 @@ END_DEFUN
 /*
  * Free all the macros (used at Zile exit).
  */
-  void
+void
 free_macros (void)
 {
   Macro *mp, *next;
@@ -231,7 +236,7 @@ free_macros (void)
  * Find a macro given its name.
  */
 Macro *
-get_macro (char *name)
+get_macro (const char *name)
 {
   Macro *mp;
   assert (name);
@@ -240,3 +245,15 @@ get_macro (char *name)
       return mp;
   return NULL;
 }
+
+/*
+ * Add macro names to a list.
+ */
+void
+add_macros_to_list (gl_list_t l, gl_listelement_compar_fn f)
+{
+  Macro *mp;
+
+  for (mp = head_mp; mp; mp = mp->next)
+    gl_sortedlist_add (l, f, xstrdup (mp->name));
+}
