@@ -23,7 +23,6 @@
 
 #include "config.h"
 
-#include <assert.h>
 #include <ctype.h>
 #include <limits.h>
 #include <locale.h>
@@ -34,16 +33,11 @@
 #include <unistd.h>
 #include <getopt.h>
 #include <signal.h>
-#if HAVE_LIBGEN_H
-#include <libgen.h>
-#endif
+#include "dirname.h"
+#include "gl_linked_list.h"
 
 #include "zile.h"
 #include "extern.h"
-#include "eval.h"
-#include "vars.h"
-
-#define ZILE_VERSION_STRING	"GNU " PACKAGE_NAME " " VERSION
 
 #define ZILE_COPYRIGHT_STRING \
   "Copyright (C) 2008 Free Software Foundation, Inc."
@@ -73,10 +67,7 @@ loop (void)
       term_redisplay ();
       term_refresh ();
 
-      thisflag = 0;
-      if (lastflag & FLAG_DEFINING_MACRO)
-        thisflag |= FLAG_DEFINING_MACRO;
-
+      thisflag = lastflag & FLAG_DEFINING_MACRO;
       key = getkey ();
       minibuf_clear ();
       process_key (key);
@@ -103,7 +94,7 @@ Type `C-h h' for help; `C-x u; to undo changes.\n\
 Type `C-h C-d' for information on getting the latest version.\n\
 Type `C-h t' for a tutorial on using " PACKAGE_NAME ".\n\
 Type `C-h s' for a sample configuration file.\n\
-Type `C-g' at any time to cancel the current operation.\n\
+Type `C-g' at any time to quit the current operation.\n\
 \n\
 `C-x' means hold the CTRL key while typing the character `x'.\n\
 `M-x' means hold the META or ALT key down while typing `x'.\n\
@@ -119,7 +110,7 @@ static void
 about_screen (void)
 {
   minibuf_write (about_minibuf_str);
-  if (!lookup_bool_variable ("inhibit-splash-screen"))
+  if (!get_variable_bool ("inhibit-splash-screen"))
     {
       show_splash_screen (about_splash_str);
       term_refresh ();
@@ -128,12 +119,12 @@ about_screen (void)
 }
 
 static void
-execute_functions (list funcs)
+execute_functions (gl_list_t funcs)
 {
-  list l;
-  for (l = list_first (funcs); l != funcs; l = list_next (l))
+  size_t i;
+  for (i = 0; i < gl_list_size (funcs); i++)
     {
-      char *func = (char *) l->item;
+      char *func = (char *) gl_list_get_at (funcs, i);
       term_redisplay ();
       if (!execute_function (func, 1))
         minibuf_error ("Function `%s' not defined", func);
@@ -155,14 +146,14 @@ setup_main_screen (int argc)
       c++;
     }
 
-  /* *scratch* and two files.  */
+  /* *scratch* and two files. */
   if (c == 3)
     {
       FUNCALL (split_window);
       switch_to_buffer (last_bp);
       FUNCALL (other_window);
     }
-  /* More than two files.  */
+  /* More than two files. */
   else if (c > 3)
     FUNCALL (list_buffers);
   else
@@ -171,12 +162,16 @@ setup_main_screen (int argc)
         {
           undo_nosave = true;
 
-          if (!lookup_bool_variable ("inhibit-splash-screen"))
-            insert_string ("\
+          if (!get_variable_bool ("inhibit-splash-screen"))
+            {
+              astr as = astr_new_cstr ("\
 This buffer is for notes you don't want to save.\n\
 If you want to create a file, visit that file with C-x C-f,\n\
 then enter the text in that file's own buffer.\n\
 \n");
+              insert_astr (as);
+              astr_delete (as);
+            }
 
           undo_nosave = false;
           cur_bp->flags &= ~BFLAG_MODIFIED;
@@ -220,9 +215,7 @@ signal_init (void)
 /* Options which take no argument have optional_argument, so that no
    arguments are signalled as extraneous, to mimic Emacs */
 struct option longopts[] = {
-  {"batch", optional_argument, NULL, 'b'},
   {"help", optional_argument, NULL, 'h'},
-  {"funcall", required_argument, NULL, 'f'},
   {"no-init-file", optional_argument, NULL, 'q'},
   {"version", optional_argument, NULL, 'v'},
   {0, 0, 0, 0}
@@ -231,45 +224,35 @@ struct option longopts[] = {
 int
 main (int argc, char **argv)
 {
-  int c, bflag = false, qflag = false;
-  list fargs = list_new ();
+  int c, qflag = false;
+  gl_list_t fargs = gl_list_create_empty (GL_LINKED_LIST,
+                                          NULL, NULL, NULL, false);
   size_t line;
 
   /* Set prog_name to executable name, if available */
-#if HAVE_BASENAME
-  if (argv[0]) {
-    char *s = xstrdup (argv[0]);
-    prog_name = xstrdup (basename (s));
-    free (s);
-  }
-#endif
+  if (argv[0])
+    prog_name = base_name (argv[0]);
 
   /* Set up Lisp environment now so it's available to files and
      expressions specified on the command-line. */
   init_search ();
-  lisp_init ();
+  init_lisp ();
   init_variables ();
+  init_eval ();
 
   opterr = 0;			/* Don't display errors for unknown options */
-  while (true)
+  for (;;)
     {
       int this_optind = optind ? optind : 1;
 
       /* Leading : so as to return ':' for a missing arg, not '?' */
-      c = getopt_long_only (argc, argv, ":bhf:qv", longopts, NULL);
+      c = getopt_long_only (argc, argv, ":h:qv", longopts, NULL);
 
       if (c == -1)
         break;
 
       switch (c)
         {
-        case 'b':
-          bflag = true;
-          qflag = true;
-          break;
-        case 'f':
-          list_append (fargs, optarg);
-          break;
         case 'q':
           qflag = true;
           break;
@@ -290,15 +273,12 @@ main (int argc, char **argv)
                    "\n"
                    "Initialization options:\n"
                    "\n"
-                   "--batch                do not do interactive display; implies -q\n"
-                   "--help                 display this help message and exit\n"
-                   "--funcall, -f FUNC     call " PACKAGE_NAME
-                   " function FUNC with no arguments\n"
-                   "--no-init-file, -q     do not load ~/." PACKAGE "\n"
-                   "--version              display version information and exit\n"
+                   "--help                  display this help message and exit\n"
+                   "--no-init-file, -q      do not load ~/." PACKAGE "\n"
+                   "--version               display version information and exit\n"
                    "\n" "Action options:\n" "\n"
-                   "FILE                   visit FILE using find-file\n"
-                   "+LINE FILE             visit FILE using find-file, then go to line LINE\n"
+                   "FILE                    visit FILE using find-file\n"
+                   "+LINE FILE              visit FILE using find-file, then go to line LINE\n"
                    "\n"
                    "Report bugs to " PACKAGE_BUGREPORT ".\n",
                    prog_name);
@@ -321,81 +301,77 @@ main (int argc, char **argv)
   setlocale (LC_ALL, "");
 
   init_bindings ();
+  init_minibuf ();
 
-  if (!bflag)
+  term_init ();
+
+  /* Create the `*scratch*' buffer. */
+  create_first_window ();
+  term_redisplay ();
+
+  /* Read settings after creating *scratch* buffer so that any
+     buffer commands won't cause a crash. */
+  if (!qflag)
     {
-      term_init ();
-
-      /* Create the `*scratch*' buffer. */
-      create_first_window ();
-      term_redisplay ();
-
-      /* Read settings after creating *scratch* buffer so that any
-         buffer commands won't cause a crash. */
-      if (!qflag)
+      astr as = get_home_dir ();
+      if (as)
         {
-          le *list;
-
-          astr as = get_home_dir ();
           astr_cat_cstr (as, "/." PACKAGE);
-          list = lisp_read_file (astr_cstr (as));
-          astr_delete (lisp_dump (list));
+          lisp_load (astr_cstr (as));
           astr_delete (as);
-          leWipe (list);
         }
-
-      /* Reinitialise the scratch buffer to catch settings */
-      init_buffer (cur_bp);
-
-      /* Open files given on the command line */
-      for (line = 1; *argv; argv++)
-        {
-          if (**argv == '+')
-            line = strtoul (*argv + 1, NULL, 10);
-          else
-            {
-              find_file (*argv);
-              ngotodown (line - 1);
-              line = 1;
-              lastflag |= FLAG_NEED_RESYNC;
-            }
-        }
-
-      /* Show the splash screen only if no files and no Lisp expression
-         or load file is specified on the command line. */
-      if (minibuf_contents == NULL && argc == 0 && list_length (fargs) == 0)
-        about_screen ();
-
-      setup_main_screen (argc);
-
-      /* Refresh minibuffer in case there's an error that couldn't be
-         written during startup */
-      if (minibuf_contents != NULL)
-        {
-          char *buf = xstrdup (minibuf_contents);
-
-          minibuf_write (buf);
-          free (buf);
-        }
-
-      execute_functions (fargs);
-      list_delete (fargs);
-
-      /* Run the main loop. */
-      loop ();
-
-      /* Tidy and close the terminal. */
-      term_tidy ();
-      term_close ();
-
-      free_bindings ();
     }
-  else if (minibuf_contents) /* if in batch mode, print any error */
-    fprintf (stderr, "%s: %s\n", prog_name, minibuf_contents);
+
+  /* Reinitialise the scratch buffer to catch settings */
+  init_buffer (cur_bp);
+
+  /* Open files given on the command line */
+  for (line = 1; *argv; argv++)
+    {
+      if (**argv == '+')
+        line = strtoul (*argv + 1, NULL, 10);
+      else
+        {
+          find_file (*argv);
+          ngotodown (line - 1);
+          line = 1;
+          lastflag |= FLAG_NEED_RESYNC;
+        }
+    }
+
+  /* Show the splash screen only if no files and no Lisp expression
+     or load file is specified on the command line. */
+  if (minibuf_contents == NULL && argc == 0 && gl_list_size (fargs) == 0)
+    about_screen ();
+
+  setup_main_screen (argc);
+
+  /* Refresh minibuffer in case there's an error that couldn't be
+     written during startup */
+  if (minibuf_contents != NULL)
+    {
+      char *buf = xstrdup (minibuf_contents);
+
+      minibuf_write (buf);
+      free (buf);
+    }
+
+  execute_functions (fargs);
+  gl_list_free (fargs);
+
+  /* Run the main loop. */
+  loop ();
+
+  /* Tidy and close the terminal. */
+  term_tidy ();
+  term_close ();
+
+  free_bindings ();
+  free_eval ();
 
   /* Free Lisp state. */
-  variableFree (mainVarList);
-  lisp_finalise ();
+  free_variables ();
+  free_lisp ();
 
   /* Free all the memory allocated. */
   free_kill_ring ();
@@ -404,8 +380,7 @@ main (int argc, char **argv)
   free_windows ();
   free_buffers ();
   free_minibuf ();
-  free_rotation_buffers ();
-  /* Don't free prog_name, as it might not have been set dynamically. */
+  free (prog_name);
 
   return 0;
 }
