@@ -29,12 +29,12 @@ Point
 make_point (size_t lineno, size_t offset)
 {
   Point pt;
-  pt.p = list_next (cur_bp->lines);
+  pt.p = cur_bp->lines->next;
   pt.n = lineno;
   pt.o = offset;
   while (lineno > 0)
     {
-      pt.p = list_next (pt.p);
+      pt.p = pt.p->next;
       lineno--;
     }
   return pt;
@@ -60,20 +60,20 @@ point_dist (Point pt1, Point pt2)
   if (cmp_point (pt1, pt2) > 0)
     swap_point (&pt1, &pt2);
 
-  for (lp = pt1.p;; lp = list_next (lp))
+  for (lp = pt1.p;; lp = lp->next)
     {
-      size += astr_len (lp->item);
+      size += astr_len (lp->text);
 
       if (lp == pt1.p)
-	size -= pt1.o;
+        size -= pt1.o;
 
       if (lp == pt2.p)
-	{
-	  size -= astr_len (lp->item) - pt2.o;
-	  break;
-	}
+        {
+          size -= astr_len (lp->text) - pt2.o;
+          break;
+        }
       else
-	size++;
+        size++;
     }
 
   return size;
@@ -100,7 +100,7 @@ Point
 point_min (void)
 {
   Point pt;
-  pt.p = list_next (cur_bp->lines);
+  pt.p = cur_bp->lines->next;
   pt.n = 0;
   pt.o = 0;
   return pt;
@@ -110,9 +110,9 @@ Point
 point_max (void)
 {
   Point pt;
-  pt.p = list_prev (cur_bp->lines);
-  pt.n = cur_bp->num_lines;
-  pt.o = astr_len (list_prev (cur_bp->lines)->item);
+  pt.p = cur_bp->lines->prev;
+  pt.n = cur_bp->last_line;
+  pt.o = astr_len (cur_bp->lines->prev->text);
   return pt;
 }
 
@@ -127,10 +127,10 @@ line_beginning_position (int count)
   pt.o = 0;
 
   count--;
-  for (; count < 0 && list_prev (pt.p) != cur_bp->lines; pt.n--, count++)
-    pt.p = list_prev (pt.p);
-  for (; count > 0 && list_next (pt.p) != cur_bp->lines; pt.n++, count--)
-    pt.p = list_next (pt.p);
+  for (; count < 0 && pt.p->prev != cur_bp->lines; pt.n--, count++)
+    pt.p = pt.p->prev;
+  for (; count > 0 && pt.p->next != cur_bp->lines; pt.n++, count--)
+    pt.p = pt.p->next;
 
   return pt;
 }
@@ -139,6 +139,28 @@ Point
 line_end_position (int count)
 {
   Point pt = line_beginning_position (count);
-  pt.o = astr_len (pt.p->item);
+  pt.o = astr_len (pt.p->text);
   return pt;
 }
+
+void
+goto_point (Point pt)
+{
+  if (cur_bp->pt.n > pt.n)
+    do
+      FUNCALL (previous_line);
+    while (cur_bp->pt.n > pt.n);
+  else if (cur_bp->pt.n < pt.n)
+    do
+      FUNCALL (next_line);
+    while (cur_bp->pt.n < pt.n);
+
+  if (cur_bp->pt.o > pt.o)
+    do
+      FUNCALL (backward_char);
+    while (cur_bp->pt.o > pt.o);
+  else if (cur_bp->pt.o < pt.o)
+    do
+      FUNCALL (forward_char);
+    while (cur_bp->pt.o < pt.o);
+}
