@@ -32,11 +32,14 @@
 
 #define NUM_REGISTERS	256
 
-static struct
+static astr regs[NUM_REGISTERS];
+
+static void
+register_free (size_t n)
 {
-  char *text;
-  size_t size;
-} regs[NUM_REGISTERS];
+  if (regs[n] != NULL)
+    astr_delete (regs[n]);
+}
 
 DEFUN ("copy-to-register", copy_to_register)
 /*+
@@ -51,32 +54,33 @@ Copy region into the user specified register.
   term_refresh ();
   reg = getkey ();
   if (reg == KBD_CANCEL)
-    return cancel ();
+    return FUNCALL (keyboard_quit);
   minibuf_clear ();
+  if (reg < 0)
+    reg = 0;
   reg %= NUM_REGISTERS;
 
-  if (warn_if_no_mark ())
-    return false;
-
-  calculate_the_region (&r);
+  if (!calculate_the_region (&r))
+    return leNIL;
 
   p = copy_text_block (r.start.n, r.start.o, r.size);
-  if (regs[reg].text != NULL)
-    free (regs[reg].text);
-  regs[reg].text = p;
-  regs[reg].size = r.size;
+  register_free ((size_t) reg);
+  regs[reg] = astr_new_cstr (p);
 
-  return true;
+  return leT;
 }
 END_DEFUN
 
-static void
-insert_register (int reg)
+static int reg;
+
+static int
+insert_register (void)
 {
-  undo_save (UNDO_REMOVE_BLOCK, cur_bp->pt, regs[reg].size, 0);
+  undo_save (UNDO_REPLACE_BLOCK, cur_bp->pt, 0, astr_len (regs[reg]));
   undo_nosave = true;
-  insert_nstring (regs[reg].text, regs[reg].size);
+  insert_astr (regs[reg]);
   undo_nosave = false;
+  return true;
 }
 
 DEFUN ("insert-register", insert_register)
@@ -85,36 +89,29 @@ Insert contents of the user specified register.
 Puts point before and mark after the inserted text.
 +*/
 {
-  int reg, uni;
-
   if (warn_if_readonly_buffer ())
-    return false;
+    return leNIL;
 
   minibuf_write ("Insert register: ");
   term_refresh ();
   reg = getkey ();
   if (reg == KBD_CANCEL)
-    return cancel ();
+    return FUNCALL (keyboard_quit);
   minibuf_clear ();
   reg %= NUM_REGISTERS;
 
-  if (regs[reg].text == NULL)
+  if (regs[reg] == NULL)
     {
       minibuf_error ("Register does not contain text");
-      return false;
+      return leNIL;
     }
 
-  set_mark_command ();
-
-  undo_save (UNDO_START_SEQUENCE, cur_bp->pt, 0, 0);
-  for (uni = 0; uni < last_uniarg; ++uni)
-    insert_register (reg);
-  undo_save (UNDO_END_SEQUENCE, cur_bp->pt, 0, 0);
-
-  exchange_point_and_mark ();
+  set_mark_interactive ();
+  execute_with_uniarg (true, uniarg, insert_register, NULL);
+  FUNCALL (exchange_point_and_mark);
   deactivate_mark ();
 
-  return true;
+  return leT;
 }
 END_DEFUN
 
@@ -126,16 +123,16 @@ write_registers_list (va_list ap GCC_UNUSED)
   bprintf ("%-8s %8s\n", "Register", "Size");
   bprintf ("%-8s %8s\n", "--------", "----");
   for (i = count = 0; i < NUM_REGISTERS; ++i)
-    if (regs[i].text != NULL)
+    if (regs[i] != NULL)
       {
-	astr as = astr_new ();
-	++count;
-	if (isprint (i))
-	  astr_afmt (as, "`%c'", i);
-	else
-	  astr_afmt (as, "`\\%o'", i);
-	bprintf ("%-8s %8d\n", astr_cstr (as), regs[i].size);
-	astr_delete (as);
+        astr as = astr_new ();
+        ++count;
+        if (isprint (i))
+          astr_afmt (as, "`%c'", i);
+        else
+          astr_afmt (as, "`\\%o'", i);
+        bprintf ("%-8s %8d\n", astr_cstr (as), astr_len (regs[i]));
+        astr_delete (as);
       }
   if (!count)
     bprintf ("No registers defined\n");
@@ -146,16 +143,15 @@ DEFUN ("list-registers", list_registers)
 List defined registers.
 +*/
 {
-  write_temp_buffer ("*Registers List*", write_registers_list);
-  return true;
+  write_temp_buffer ("*Registers List*", true, write_registers_list);
+  return leT;
 }
 END_DEFUN
 
 void
 free_registers (void)
 {
-  int i;
+  size_t i;
   for (i = 0; i < NUM_REGISTERS; ++i)
-    if (regs[i].text != NULL)
-      free (regs[i].text);
+    register_free (i);
 }
