@@ -24,7 +24,6 @@
 
 #include "config.h"
 
-#include <assert.h>
 #include <limits.h>
 #include <ctype.h>
 #include <stdarg.h>
@@ -77,7 +76,7 @@ no_upper (const char *s, size_t len, int regex)
 static const char *
 fold_table (const char *s, int regex)
 {
-  if (lookup_bool_variable ("case-fold-search")
+  if (get_variable_bool ("case-fold-search")
       && no_upper (s, strlen (s), regex))
     return downcase;
   else
@@ -141,8 +140,8 @@ re_find_substr (const char *s1, size_t s1size,
   old_syntax = re_set_syntax (RE_SYNTAX_EMACS);
 
   search_regs.num_regs = 1;
-  search_regs.start = xzalloc (sizeof (regoff_t));
-  search_regs.end = xzalloc (sizeof (regoff_t));
+  search_regs.start = XZALLOC (regoff_t);
+  search_regs.end = XZALLOC (regoff_t);
 
   /* translate table is never written to, so this cast is safe */
   pattern.translate = (unsigned char *) translate;
@@ -211,24 +210,24 @@ search_forward (Line * startp, size_t starto, const char *s, int regexp)
   if (s2size < 1)
     return false;
 
-  for (lp = startp; lp != cur_bp->lines; lp = list_next (lp))
+  for (lp = startp; lp != cur_bp->lines; lp = lp->next)
     {
       if (lp == startp)
 	{
-	  sp = astr_char (lp->item, (ptrdiff_t) starto);
-	  s1size = astr_len (lp->item) - starto;
+	  sp = astr_char (lp->text, (ptrdiff_t) starto);
+	  s1size = astr_len (lp->text) - starto;
 	}
       else
 	{
-	  sp = astr_cstr (lp->item);
-	  s1size = astr_len (lp->item);
+	  sp = astr_cstr (lp->text);
+	  s1size = astr_len (lp->text);
 	}
       if (s1size < 1)
 	continue;
 
       if (regexp)
 	sp2 = re_find_substr (sp, s1size, s, s2size,
-			      sp == astr_cstr (lp->item), true, false,
+			      sp == astr_cstr (lp->text), true, false,
 			      translate);
       else
 	sp2 = find_substr (sp, s1size, s, s2size, translate);
@@ -236,7 +235,7 @@ search_forward (Line * startp, size_t starto, const char *s, int regexp)
       if (sp2 != NULL)
 	{
 	  goto_linep (lp);
-	  cur_bp->pt.o = sp2 - astr_cstr (lp->item);
+	  cur_bp->pt.o = sp2 - astr_cstr (lp->text);
 	  return true;
 	}
     }
@@ -254,19 +253,19 @@ search_backward (Line * startp, size_t starto, const char *s, int regexp)
   if (ssize < 1)
     return false;
 
-  for (lp = startp; lp != cur_bp->lines; lp = list_prev (lp))
+  for (lp = startp; lp != cur_bp->lines; lp = lp->prev)
     {
-      sp = astr_cstr (lp->item);
+      sp = astr_cstr (lp->text);
       if (lp == startp)
 	s1size = starto;
       else
-	s1size = astr_len (lp->item);
+	s1size = astr_len (lp->text);
       if (s1size < 1)
 	continue;
 
       if (regexp)
 	sp2 = re_find_substr (sp, s1size, s, ssize,
-			      true, s1size == astr_len (lp->item), true,
+			      true, s1size == astr_len (lp->text), true,
 			      translate);
       else
 	sp2 = rfind_substr (sp, s1size, s, ssize, translate);
@@ -274,7 +273,7 @@ search_backward (Line * startp, size_t starto, const char *s, int regexp)
       if (sp2 != NULL)
 	{
 	  goto_linep (lp);
-	  cur_bp->pt.o = sp2 - astr_cstr (lp->item);
+	  cur_bp->pt.o = sp2 - astr_cstr (lp->text);
 	  return true;
 	}
     }
@@ -284,29 +283,41 @@ search_backward (Line * startp, size_t starto, const char *s, int regexp)
 
 static char *last_search = NULL;
 
+typedef int (*Searcher) (Line * startp, size_t starto, const char *s, int regexp);
+
+static le *
+search (Searcher searcher, bool regexp, const char *search_msg)
+{
+  char *ms = minibuf_read (search_msg, last_search);
+
+  if (ms == NULL)
+    return FUNCALL (keyboard_quit);
+  if (ms[0] == '\0')
+    {
+      free (ms);
+      return leNIL;
+    }
+
+  free (last_search);
+  last_search = xstrdup (ms);
+
+  if (!searcher (cur_bp->pt.p, cur_bp->pt.o, ms, regexp))
+    {
+      minibuf_error ("Search failed: \"%s\"", ms);
+      free (ms);
+      return leNIL;
+    }
+
+  free (ms);
+  return leT;
+}
+
 DEFUN ("search-forward", search_forward)
 /*+
 Search forward from point for the user specified text.
 +*/
 {
-  char *ms = minibuf_read ("Search: ", last_search);
-
-  if (ms == NULL)
-    return cancel ();
-  if (ms[0] == '\0')
-    return false;
-
-  if (last_search != NULL)
-    free (last_search);
-  last_search = xstrdup (ms);
-
-  if (!search_forward (cur_bp->pt.p, cur_bp->pt.o, ms, false))
-    {
-      minibuf_error ("Failing search: `%s'", ms);
-      return false;
-    }
-
-  return true;
+  return search (search_forward, false, "Search: ");
 }
 END_DEFUN
 
@@ -315,24 +326,7 @@ DEFUN ("search-backward", search_backward)
 Search backward from point for the user specified text.
 +*/
 {
-  char *ms = minibuf_read ("Search backward: ", last_search);
-
-  if (ms == NULL)
-    return cancel ();
-  if (ms[0] == '\0')
-    return false;
-
-  if (last_search != NULL)
-    free (last_search);
-  last_search = xstrdup (ms);
-
-  if (!search_backward (cur_bp->pt.p, cur_bp->pt.o, ms, false))
-    {
-      minibuf_error ("Failing search: `%s'", ms);
-      return false;
-    }
-
-  return true;
+  return search (search_backward, false, "Search backward: ");
 }
 END_DEFUN
 
@@ -341,24 +335,7 @@ DEFUN ("search-forward-regexp", search_forward_regexp)
 Search forward from point for regular expression REGEXP.
 +*/
 {
-  char *ms = minibuf_read ("Regexp search: ", last_search);
-
-  if (ms == NULL)
-    return cancel ();
-  if (ms[0] == '\0')
-    return false;
-
-  if (last_search != NULL)
-    free (last_search);
-  last_search = xstrdup (ms);
-
-  if (!search_forward (cur_bp->pt.p, cur_bp->pt.o, ms, true))
-    {
-      minibuf_error ("Failing regexp search: `%s'", ms);
-      return false;
-    }
-
-  return true;
+  return search (search_forward, true, "RE search: ");
 }
 END_DEFUN
 
@@ -367,24 +344,7 @@ DEFUN ("search-backward-regexp", search_backward_regexp)
 Search backward from point for match for regular expression REGEXP.
 +*/
 {
-  char *ms = minibuf_read ("Regexp search backward: ", last_search);
-
-  if (ms == NULL)
-    return cancel ();
-  if (ms[0] == '\0')
-    return false;
-
-  if (last_search != NULL)
-    free (last_search);
-  last_search = xstrdup (ms);
-
-  if (!search_backward (cur_bp->pt.p, cur_bp->pt.o, ms, true))
-    {
-      minibuf_error ("Failing regexp search backward: `%s'", ms);
-      return false;
-    }
-
-  return true;
+  return search (search_backward, true, "RE search backward: ");
 }
 END_DEFUN
 
@@ -393,7 +353,7 @@ END_DEFUN
 /*
  * Incremental search engine.
  */
-  static int
+static le *
 isearch (int dir, int regexp)
 {
   int c;
@@ -443,7 +403,7 @@ isearch (int dir, int regexp)
 	  thisflag |= FLAG_NEED_RESYNC;
 
 	  /* Quit. */
-	  cancel ();
+	  FUNCALL (keyboard_quit);
 
 	  /* Restore old mark position. */
 	  if (cur_bp->mark)
@@ -478,8 +438,7 @@ isearch (int dir, int regexp)
 	      /* Find next match. */
 	      cur = cur_bp->pt;
 	      /* Save search string. */
-	      if (last_search != NULL)
-		free (last_search);
+              free (last_search);
 	      last_search = xstrdup (astr_cstr (pattern));
 	    }
 	  else if (last_search != NULL)
@@ -513,8 +472,7 @@ isearch (int dir, int regexp)
 		  cur_bp->mark->pt = start;
 
 		  /* Save search string. */
-		  if (last_search != NULL)
-		    free (last_search);
+                  free (last_search);
 		  last_search = xstrdup (astr_cstr (pattern));
 
 		  minibuf_write ("Mark saved when search started");
@@ -553,7 +511,7 @@ isearch (int dir, int regexp)
   if (old_mark)
     free_marker (old_mark);
 
-  return true;
+  return leT;
 }
 
 DEFUN ("isearch-forward", isearch_forward)
@@ -608,48 +566,6 @@ is treated as a regexp.  See M-x isearch-forward for more info.
 }
 END_DEFUN
 
-DEFUN ("replace-string", replace_string)
-/*+
-Replace occurrences of a string with other text.
-+*/
-{
-  int count = 0, find_no_upper;
-  size_t find_len, repl_len;
-  char *find = minibuf_read ("Replace string: ", "");
-  char *repl;
-
-  if (find == NULL)
-    return cancel ();
-  if (find[0] == '\0')
-    return false;
-  find_len = strlen (find);
-  find_no_upper = no_upper (find, find_len, false);
-
-  repl = minibuf_read ("Replace `%s' with: ", "", find);
-  if (repl == NULL)
-    return cancel ();
-  repl_len = strlen (repl);
-
-  while (search_forward (cur_bp->pt.p, cur_bp->pt.o, find, false))
-    {
-      ++count;
-      undo_save (UNDO_REPLACE_BLOCK,
-		 make_point (cur_bp->pt.n,
-			     cur_bp->pt.o - find_len), find_len, repl_len);
-      line_replace_text (&cur_bp->pt.p, cur_bp->pt.o - find_len,
-			 find_len, repl, repl_len, find_no_upper);
-    }
-
-  if (thisflag & FLAG_NEED_RESYNC)
-    resync_redisplay ();
-  term_redisplay ();
-
-  minibuf_write ("Replaced %d occurrences", count);
-
-  return true;
-}
-END_DEFUN
-
 DEFUN ("query-replace", query_replace)
 /*+
 Replace occurrences of a string with other text.
@@ -663,15 +579,21 @@ what to do with it.
   char *repl;
 
   if (find == NULL)
-    return cancel ();
+    return FUNCALL (keyboard_quit);
   if (*find == '\0')
-    return false;
+    {
+      free (find);
+      return leNIL;
+    }
   find_len = strlen (find);
   find_no_upper = no_upper (find, find_len, false);
 
   repl = minibuf_read ("Query replace `%s' with: ", "", find);
   if (repl == NULL)
-    return cancel ();
+    {
+      free (find);
+      return FUNCALL (keyboard_quit);
+    }
   repl_len = strlen (repl);
 
   /* Spaghetti code follows... :-( */
@@ -700,7 +622,7 @@ what to do with it.
 	  switch (c)
 	    {
 	    case KBD_CANCEL:	/* C-g */
-	      return cancel ();
+	      return FUNCALL (keyboard_quit);
 	    case 'q':		/* Quit immediately. */
 	      goto endoffunc;
 	    case '.':		/* Replace and quit. */
@@ -733,12 +655,15 @@ what to do with it.
     }
 
 endoffunc:
+  free (find);
+  free (repl);
+
   if (thisflag & FLAG_NEED_RESYNC)
     resync_redisplay ();
   term_redisplay ();
 
   minibuf_write ("Replaced %d occurrences", count);
 
-  return true;
+  return leT;
 }
 END_DEFUN
