@@ -32,24 +32,11 @@
 #include "config.h"
 #include "extern.h"
 
-static int initted = false;
 static size_t width = 0, height = 0;
 static size_t cur_tab_width;
 static size_t cur_topline;
 static size_t point_screen_column;
 
-int
-term_initted (void)
-{
-  return initted;
-}
-
-void
-term_set_initted (void)
-{
-  initted = true;
-}
-
 size_t
 term_width (void)
 {
@@ -81,7 +68,7 @@ make_char_printable (char **buf, size_t c)
 }
 
 static void
-outch (int c, Font font, size_t * x)
+outch (int c, size_t font, size_t * x)
 {
   int j, w;
   char *buf;
@@ -93,7 +80,7 @@ outch (int c, Font font, size_t * x)
 
   if (c == '\t')
     for (w = cur_tab_width - *x % cur_tab_width; w > 0 && *x < term_width ();
-	 w--)
+         w--)
       term_addch (' '), ++(*x);
   else if (isprint (c))
     term_addch (c), ++(*x);
@@ -101,7 +88,7 @@ outch (int c, Font font, size_t * x)
     {
       j = make_char_printable (&buf, (size_t) c);
       for (w = 0; w < j && *x < term_width (); ++w)
-	term_addch (buf[w]), ++(*x);
+        term_addch (buf[w]), ++(*x);
       free (buf);
     }
 
@@ -114,24 +101,24 @@ in_region (size_t lineno, size_t x, Region * r)
   if (lineno >= r->start.n && lineno <= r->end.n)
     {
       if (r->start.n == r->end.n)
-	{
-	  if (x >= r->start.o && x < r->end.o)
-	    return true;
-	}
+        {
+          if (x >= r->start.o && x < r->end.o)
+            return true;
+        }
       else if (lineno == r->start.n)
-	{
-	  if (x >= r->start.o)
-	    return true;
-	}
+        {
+          if (x >= r->start.o)
+            return true;
+        }
       else if (lineno == r->end.n)
-	{
-	  if (x < r->end.o)
-	    return true;
-	}
+        {
+          if (x < r->end.o)
+            return true;
+        }
       else
-	{
-	  return true;
-	}
+        {
+          return true;
+        }
     }
 
   return false;
@@ -139,7 +126,7 @@ in_region (size_t lineno, size_t x, Region * r)
 
 static void
 draw_end_of_line (size_t line, Window * wp, size_t lineno, Region * r,
-		  int highlight, size_t x, size_t i)
+                  int highlight, size_t x, size_t i)
 {
   if (x >= term_width ())
     {
@@ -149,28 +136,28 @@ draw_end_of_line (size_t line, Window * wp, size_t lineno, Region * r,
   else if (highlight)
     {
       for (; x < wp->ewidth; ++i)
-	{
-	  if (in_region (lineno, i, r))
-	    outch (' ', FONT_REVERSE, &x);
-	  else
-	    x++;
-	}
+        {
+          if (in_region (lineno, i, r))
+            outch (' ', FONT_REVERSE, &x);
+          else
+            x++;
+        }
     }
 }
 
 static void
 draw_line (size_t line, size_t startcol, Window * wp, Line * lp,
-	   size_t lineno, Region * r, int highlight)
+           size_t lineno, Region * r, int highlight)
 {
   size_t x, i;
 
   term_move (line, 0);
-  for (x = 0, i = startcol; i < astr_len (lp->item) && x < wp->ewidth; i++)
+  for (x = 0, i = startcol; i < astr_len (lp->text) && x < wp->ewidth; i++)
     {
       if (highlight && in_region (lineno, i, r))
-	outch (*astr_char (lp->item, (ptrdiff_t) i), FONT_REVERSE, &x);
+        outch (*astr_char (lp->text, (ptrdiff_t) i), FONT_REVERSE, &x);
       else
-	outch (*astr_char (lp->item, (ptrdiff_t) i), FONT_NORMAL, &x);
+        outch (*astr_char (lp->text, (ptrdiff_t) i), FONT_NORMAL, &x);
     }
 
   draw_end_of_line (line, wp, lineno, r, highlight, x, i);
@@ -180,10 +167,10 @@ static void
 calculate_highlight_region (Window * wp, Region * r, int *highlight)
 {
   if ((wp != cur_wp
-       && !lookup_bool_variable ("highlight-nonselected-windows"))
+       && !get_variable_bool ("highlight-nonselected-windows"))
       || (!wp->bp->mark)
       || (!transient_mark_mode ())
-      || (transient_mark_mode () && !(wp->bp->mark_active)))
+      || (transient_mark_mode () && !(wp->bp->flags & BFLAG_MARK)))
     {
       *highlight = false;
       return;
@@ -209,8 +196,8 @@ draw_window (size_t topline, Window * wp)
 
   /* Find the first line to display on the first screen line. */
   for (lp = pt.p, lineno = pt.n, i = wp->topdelta;
-       i > 0 && list_prev (lp) != wp->bp->lines;
-       lp = list_prev (lp), --i, --lineno)
+       i > 0 && lp->prev != wp->bp->lines;
+       lp = lp->prev, --i, --lineno)
     ;
 
   cur_tab_width = tab_width (wp->bp);
@@ -224,19 +211,19 @@ draw_window (size_t topline, Window * wp)
 
       /* If at the end of the buffer, don't write any text. */
       if (lp == wp->bp->lines)
-	continue;
+        continue;
 
       startcol = wp->start_column;
 
       draw_line (i, startcol, wp, lp, lineno, &r, highlight);
 
       if (wp->start_column > 0)
-	{
-	  term_move (i, 0);
-	  term_addch ('$');
-	}
+        {
+          term_move (i, 0);
+          term_addch ('$');
+        }
 
-      lp = list_next (lp);
+      lp = lp->next;
     }
 }
 
@@ -271,33 +258,33 @@ calculate_start_column (Window * wp)
   char *buf, *rp, *lp, *p;
   Point pt = window_pt (wp);
 
-  rp = astr_char (pt.p->item, (ptrdiff_t) pt.o);
+  rp = astr_char (pt.p->text, (ptrdiff_t) pt.o);
   rpfact = pt.o / (wp->ewidth / 3);
 
-  for (lp = rp; lp >= astr_cstr (pt.p->item); --lp)
+  for (lp = rp; lp >= astr_cstr (pt.p->text); --lp)
     {
       for (col = 0, p = lp; p < rp; ++p)
-	if (*p == '\t')
-	  {
-	    col |= t - 1;
-	    ++col;
-	  }
-	else if (isprint ((int) *p))
-	  ++col;
-	else
-	  {
-	    col += make_char_printable (&buf, (size_t) * p);
-	    free (buf);
-	  }
+        if (*p == '\t')
+          {
+            col |= t - 1;
+            ++col;
+          }
+        else if (isprint ((int) *p))
+          ++col;
+        else
+          {
+            col += make_char_printable (&buf, (size_t) * p);
+            free (buf);
+          }
 
-      lpfact = (lp - astr_cstr (pt.p->item)) / (wp->ewidth / 3);
+      lpfact = (lp - astr_cstr (pt.p->text)) / (wp->ewidth / 3);
 
       if (col >= wp->ewidth - 1 || lpfact < (rpfact - 2))
-	{
-	  wp->start_column = lp + 1 - astr_cstr (pt.p->item);
-	  point_screen_column = lastcol;
-	  return;
-	}
+        {
+          wp->start_column = lp + 1 - astr_cstr (pt.p->text);
+          point_screen_column = lastcol;
+          return;
+        }
 
       lastcol = col;
     }
@@ -311,14 +298,14 @@ make_screen_pos (Window * wp, char **buf)
 {
   Point pt = window_pt (wp);
 
-  if (wp->bp->num_lines <= wp->eheight && wp->topdelta == pt.n)
+  if (wp->bp->last_line <= wp->eheight && wp->topdelta == pt.n)
     xasprintf (buf, "All");
   else if (pt.n == wp->topdelta)
     xasprintf (buf, "Top");
-  else if (pt.n + (wp->eheight - wp->topdelta) > wp->bp->num_lines)
+  else if (pt.n + (wp->eheight - wp->topdelta) > wp->bp->last_line)
     xasprintf (buf, "Bot");
   else
-    xasprintf (buf, "%2d%%", (int) ((float) pt.n / wp->bp->num_lines * 100));
+    xasprintf (buf, "%2d%%", (int) ((float) pt.n / wp->bp->last_line * 100));
 
   return *buf;
 }
@@ -345,10 +332,11 @@ draw_status_line (size_t line, Window * wp)
     eol_type = ":";
 
   term_move (line, 0);
-  bs = astr_afmt (astr_new (), "(%d,%d)", pt.n + 1, get_goalc_wp (wp));
+  bs = astr_afmt (astr_new (), "(%d,%d)", pt.n + 1,
+                  get_goalc_bp (wp->bp, window_pt (wp)));
   as = astr_afmt (astr_new (), "--%s%2s  %-15s   %s %-9s (Text",
-		  eol_type, make_mode_line_flags (wp), wp->bp->name,
-		  make_screen_pos (wp, &buf), astr_cstr (bs));
+                  eol_type, make_mode_line_flags (wp), wp->bp->name,
+                  make_screen_pos (wp, &buf), astr_cstr (bs));
   free (buf);
   astr_delete (bs);
 
@@ -381,24 +369,19 @@ term_redisplay (void)
   for (wp = head_wp; wp != NULL; wp = wp->next)
     {
       if (wp == cur_wp)
-	cur_topline = topline;
+        cur_topline = topline;
 
       draw_window (topline, wp);
 
       /* Draw the status line only if there is available space after the
-	 buffer text space. */
+         buffer text space. */
       if (wp->fheight - wp->eheight > 0)
-	draw_status_line (topline + wp->eheight, wp);
+        draw_status_line (topline + wp->eheight, wp);
 
       topline += wp->fheight;
     }
 
-  term_redraw_cursor ();
-}
-
-void
-term_redraw_cursor (void)
-{
+  /* Redraw cursor. */
   term_move (cur_topline + cur_wp->topdelta, point_screen_column);
 }
 
