@@ -22,7 +22,6 @@
 
 #include "config.h"
 
-#include <assert.h>
 #include <stdarg.h>
 #include <stdio.h>
 #include <stdlib.h>
@@ -31,54 +30,63 @@
 #include "zile.h"
 #include "extern.h"
 
-/* This variable is set to true when the `undo_save()' function should not
-   register the undo information. */
+/*
+ * Undo action
+ */
+struct Undo
+{
+  /* Next undo delta in list. */
+  Undo *next;
+
+  /* The type of undo delta. */
+  int type;
+
+  /* Where the undo delta need to be applied.
+     Warning!: Do not use the "pt.p" field. */
+  Point pt;
+
+  /* Flag indicating that reverting this undo leaves the buffer
+     in an unchanged state. */
+  int unchanged;
+
+  /* The block to insert. */
+  struct
+  {
+    char *text;
+    size_t osize;		/* Original size. */
+    size_t size;		/* New block size. */
+  } block;
+};
+
+/* Setting this variable to true stops undo_save saving the given
+   information. */
 int undo_nosave = false;
 
-/* This variable is set to true when the undo is in execution. */
+/* This variable is set to true when an undo is in execution. */
 static int doing_undo = false;
 
 /*
  * Save a reverse delta for doing undo.
  */
 void
-undo_save (int type, Point pt, size_t arg1, size_t arg2)
+undo_save (int type, Point pt, size_t osize, size_t size)
 {
   Undo *up;
 
   if (cur_bp->flags & BFLAG_NOUNDO || undo_nosave)
     return;
 
-  up = (Undo *) xzalloc (sizeof (Undo));
-
+  up = (Undo *) XZALLOC (Undo);
   up->type = type;
   up->pt = pt;
-
-  /* If the buffer is currently unchanged, record the fact. */
   if (!(cur_bp->flags & BFLAG_MODIFIED))
     up->unchanged = true;
 
-  switch (type)
+  if (type == UNDO_REPLACE_BLOCK)
     {
-    case UNDO_REPLACE_CHAR:
-    case UNDO_INTERCALATE_CHAR:
-      up->delta.c = (char) arg1;
-      break;
-    case UNDO_INSERT_BLOCK:
-      up->delta.block.size = arg1;
-      up->delta.block.text = copy_text_block (pt.n, pt.o, arg1);
-      break;
-    case UNDO_REPLACE_BLOCK:
-      up->delta.block.osize = arg1;
-      up->delta.block.size = arg2;
-      up->delta.block.text = copy_text_block (pt.n, pt.o, arg1);
-      break;
-    case UNDO_REMOVE_BLOCK:
-      up->delta.block.size = arg1;
-      break;
-    case UNDO_START_SEQUENCE:
-    case UNDO_END_SEQUENCE:
-      break;
+      up->block.osize = osize;
+      up->block.size = size;
+      up->block.text = copy_text_block (pt.n, pt.o, osize);
     }
 
   up->next = cur_bp->last_undop;
@@ -103,9 +111,7 @@ revert_action (Undo * up)
       undo_save (UNDO_START_SEQUENCE, up->pt, 0, 0);
       up = up->next;
       while (up->type != UNDO_START_SEQUENCE)
-	{
-	  up = revert_action (up);
-	}
+        up = revert_action (up);
       undo_save (UNDO_END_SEQUENCE, up->pt, 0, 0);
       goto_point (up->pt);
       return up->next;
@@ -113,54 +119,15 @@ revert_action (Undo * up)
 
   goto_point (up->pt);
 
-  switch (up->type)
+  if (up->type == UNDO_REPLACE_BLOCK)
     {
-    case UNDO_INTERCALATE_CHAR:
-      if (up->delta.c == '\n')
-	intercalate_newline ();
-      else
-	intercalate_char (up->delta.c);
-      break;
-    case UNDO_INSERT_BLOCK:
-      undo_save (UNDO_REMOVE_BLOCK, up->pt, up->delta.block.size, 0);
-      undo_nosave = true;
-      for (i = 0; i < up->delta.block.size; ++i)
-	if (up->delta.block.text[i] != '\n')
-	  insert_char (up->delta.block.text[i]);
-	else
-	  insert_newline ();
-      undo_nosave = false;
-      break;
-    case UNDO_REMOVE_CHAR:
-      delete_char ();
-      break;
-    case UNDO_REMOVE_BLOCK:
-      undo_save (UNDO_INSERT_BLOCK, up->pt, up->delta.block.size, 0);
-      undo_nosave = true;
-      for (i = 0; i < up->delta.block.size; ++i)
-	delete_char ();
-      undo_nosave = false;
-      break;
-    case UNDO_REPLACE_CHAR:
-      undo_save (UNDO_REPLACE_CHAR, up->pt,
-		 (size_t) (*astr_char (cur_bp->pt.p->item,
-				       (ptrdiff_t) up->pt.o)), 0);
-      *astr_char (cur_bp->pt.p->item, (ptrdiff_t) up->pt.o) = up->delta.c;
-      cur_bp->flags |= BFLAG_MODIFIED;
-      break;
-    case UNDO_REPLACE_BLOCK:
       undo_save (UNDO_REPLACE_BLOCK, up->pt,
-		 up->delta.block.size, up->delta.block.osize);
+                 up->block.size, up->block.osize);
       undo_nosave = true;
-      for (i = 0; i < up->delta.block.size; ++i)
-	delete_char ();
-      for (i = 0; i < up->delta.block.osize; ++i)
-	if (up->delta.block.text[i] != '\n')
-	  insert_char (up->delta.block.text[i]);
-	else
-	  insert_newline ();
+      for (i = 0; i < up->block.size; ++i)
+        delete_char ();
+      insert_nstring (up->block.text, up->block.osize);
       undo_nosave = false;
-      break;
     }
 
   doing_undo = false;
@@ -182,23 +149,47 @@ Repeat this command to undo more changes.
   if (cur_bp->flags & BFLAG_NOUNDO)
     {
       minibuf_error ("Undo disabled in this buffer");
-      return false;
+      return leNIL;
     }
 
   if (warn_if_readonly_buffer ())
-    return false;
+    return leNIL;
 
   if (cur_bp->next_undop == NULL)
     {
       minibuf_error ("No further undo information");
       cur_bp->next_undop = cur_bp->last_undop;
-      return false;
+      return leNIL;
     }
 
   cur_bp->next_undop = revert_action (cur_bp->next_undop);
-
   minibuf_write ("Undo!");
-
-  return true;
+  return leT;
 }
 END_DEFUN
+
+void
+free_undo (Undo *up)
+{
+  while (up != NULL)
+    {
+      Undo *next_up = up->next;
+      if (up->type == UNDO_REPLACE_BLOCK)
+        free (up->block.text);
+      free (up);
+      up = next_up;
+    }
+}
+
+/*
+ * Set unchanged flags to false except for the last undo action (the
+ * argument), which is set to true.
+ */
+void
+undo_set_unchanged (Undo *up)
+{
+  if (up)
+    up->unchanged = true;
+  for (up = up->next; up; up = up->next)
+    up->unchanged = false;
+}
