@@ -34,18 +34,13 @@
 static Window *
 window_new (void)
 {
-  Window *wp;
-
-  wp = (Window *) xzalloc (sizeof (Window));
-  memset (wp, 0, sizeof (Window));
-
-  return wp;
+  return (Window *) XZALLOC (Window);
 }
 
 /*
- * Free the window allocated memory.
+ * Free a window's allocated memory.
  */
-void
+static void
 free_window (Window * wp)
 {
   if (wp->saved_pt)
@@ -69,8 +64,9 @@ free_windows (void)
     }
 }
 
-/* Set the current window and his buffer as the current buffer.  */
-
+/*
+ * Set the current window and his buffer as the current buffer.
+ */
 void
 set_current_window (Window * wp)
 {
@@ -108,8 +104,8 @@ Both windows display the same buffer now current.
   if (cur_wp->fheight < 4)
     {
       minibuf_error ("Window height %d too small for splitting",
-		     cur_wp->fheight);
-      return false;
+                     cur_wp->fheight);
+      return leNIL;
     }
 
   newwp = window_new ();
@@ -126,7 +122,7 @@ Both windows display the same buffer now current.
   newwp->next = cur_wp->next;
   cur_wp->next = newwp;
 
-  return true;
+  return leT;
 }
 END_DEFUN
 
@@ -140,10 +136,10 @@ delete_window (Window * del_wp)
   else
     for (wp = head_wp; wp != NULL; wp = wp->next)
       if (wp->next == cur_wp)
-	{
-	  wp->next = wp->next->next;
-	  break;
-	}
+        {
+          wp->next = wp->next->next;
+          break;
+        }
 
   wp->fheight += cur_wp->fheight;
   wp->eheight += cur_wp->eheight + 1;
@@ -160,12 +156,12 @@ Remove the current window from the screen.
   if (cur_wp == head_wp && cur_wp->next == NULL)
     {
       minibuf_error ("Attempt to delete sole ordinary window");
-      return false;
+      return leNIL;
     }
 
   delete_window (cur_wp);
 
-  return true;
+  return leT;
 }
 END_DEFUN
 
@@ -177,20 +173,20 @@ Make current window one line bigger.
   Window *wp;
 
   if (cur_wp == head_wp && cur_wp->next == NULL)
-    return false;
+    return leNIL;
 
   wp = cur_wp->next;
   if (wp == NULL || wp->fheight < 3)
     for (wp = head_wp; wp != NULL; wp = wp->next)
       if (wp->next == cur_wp)
-	{
-	  if (wp->fheight < 3)
-	    return false;
-	  break;
-	}
+        {
+          if (wp->fheight < 3)
+            return leNIL;
+          break;
+        }
 
   if (cur_wp == head_wp && cur_wp->next->fheight < 3)
-    return false;
+    return leNIL;
 
   --wp->fheight;
   --wp->eheight;
@@ -199,7 +195,7 @@ Make current window one line bigger.
   ++cur_wp->fheight;
   ++cur_wp->eheight;
 
-  return true;
+  return leT;
 }
 END_DEFUN
 
@@ -211,16 +207,14 @@ Make current window one line smaller.
   Window *wp;
 
   if ((cur_wp == head_wp && cur_wp->next == NULL) || cur_wp->fheight < 3)
-    return false;
+    return leNIL;
 
   wp = cur_wp->next;
   if (wp == NULL)
     {
       for (wp = head_wp; wp != NULL; wp = wp->next)
-	{
-	  if (wp->next == cur_wp)
-	    break;
-	}
+        if (wp->next == cur_wp)
+          break;
     }
 
   ++wp->fheight;
@@ -230,7 +224,7 @@ Make current window one line smaller.
   if (cur_wp->topdelta >= cur_wp->eheight)
     recenter (cur_wp);
 
-  return true;
+  return leT;
 }
 END_DEFUN
 
@@ -263,7 +257,7 @@ Make the selected window fill the screen.
     {
       nextwp = wp->next;
       if (wp != cur_wp)
-	free_window (wp);
+        free_window (wp);
     }
 
   cur_wp->fwidth = cur_wp->ewidth = term_width ();
@@ -274,7 +268,7 @@ Make the selected window fill the screen.
   cur_wp->next = NULL;
   head_wp = cur_wp;
 
-  return true;
+  return leT;
 }
 END_DEFUN
 
@@ -286,15 +280,15 @@ This command selects the window one step away in that order.
 +*/
 {
   set_current_window ((cur_wp->next != NULL) ? cur_wp->next : head_wp);
-  return true;
+  return leT;
 }
 END_DEFUN
 
 /*
- * This function is called once in main(), for creating
+ * This function is called once in main, for creating
  * the scratch buffer.
  */
-  void
+void
 create_first_window (void)
 {
   Window *wp;
@@ -344,8 +338,8 @@ window_pt (Window * wp)
   else
     {
       if (wp->saved_pt != NULL)
-	return wp->saved_pt->pt;
+        return wp->saved_pt->pt;
       else
-	return wp->bp->pt;
+        return wp->bp->pt;
     }
 }
